@@ -15,10 +15,11 @@ module Hledger.Data (
                module Hledger.Data.Dates,
                module Hledger.Data.Journal,
                module Hledger.Data.Ledger,
+               module Hledger.Data.Period,
                module Hledger.Data.Posting,
                module Hledger.Data.RawOptions,
                module Hledger.Data.StringFormat,
-               module Hledger.Data.TimeLog,
+               module Hledger.Data.Timeclock,
                module Hledger.Data.Transaction,
                module Hledger.Data.Types,
                tests_Hledger_Data
@@ -33,10 +34,11 @@ import Hledger.Data.Commodity
 import Hledger.Data.Dates
 import Hledger.Data.Journal
 import Hledger.Data.Ledger
+import Hledger.Data.Period
 import Hledger.Data.Posting
 import Hledger.Data.RawOptions
 import Hledger.Data.StringFormat
-import Hledger.Data.TimeLog
+import Hledger.Data.Timeclock
 import Hledger.Data.Transaction
 import Hledger.Data.Types
 
@@ -47,13 +49,12 @@ tests_Hledger_Data = TestList
     ,tests_Hledger_Data_AccountName
     ,tests_Hledger_Data_Amount
     ,tests_Hledger_Data_Commodity
-    ,tests_Hledger_Data_Dates
     ,tests_Hledger_Data_Journal
     ,tests_Hledger_Data_Ledger
     ,tests_Hledger_Data_Posting
     -- ,tests_Hledger_Data_RawOptions
     -- ,tests_Hledger_Data_StringFormat
-    ,tests_Hledger_Data_TimeLog
+    ,tests_Hledger_Data_Timeclock
     ,tests_Hledger_Data_Transaction
     -- ,tests_Hledger_Data_Types
     ]
