@@ -1,7 +1,6 @@
-{-# LANGUAGE CPP, StandaloneDeriving, RecordWildCards #-}
 {-|
 A simple 'Amount' is some quantity of money, shares, or anything else.
-It has a (possibly null) 'Commodity' and a numeric quantity:
+It has a (possibly null) 'CommoditySymbol' and a numeric quantity:
 
 @
   $1
@@ -41,6 +40,8 @@ exchange rates.
 
 -}
 
+{-# LANGUAGE CPP, StandaloneDeriving, RecordWildCards, OverloadedStrings #-}
+
 module Hledger.Data.Amount (
   -- * Amount
   amount,
@@ -101,15 +102,13 @@ module Hledger.Data.Amount (
 ) where
 
 import Data.Char (isDigit)
-#ifdef DOUBLE
-roundTo = flip const
-#else
 import Data.Decimal (roundTo)
-#endif
 import Data.Function (on)
 import Data.List
 import Data.Map (findWithDefault)
 import Data.Maybe
+-- import Data.Text (Text)
+import qualified Data.Text as T
 import Test.HUnit
 import Text.Printf
 import qualified Data.Map as M
@@ -180,7 +179,7 @@ similarAmountsOp op Amount{acommodity=_,  aquantity=q1, astyle=AmountStyle{aspre
 
 -- | Convert an amount to the specified commodity, ignoring and discarding
 -- any assigned prices and assuming an exchange rate of 1.
-amountWithCommodity :: Commodity -> Amount -> Amount
+amountWithCommodity :: CommoditySymbol -> Amount -> Amount
 amountWithCommodity c a = a{acommodity=c, aprice=NoPrice}
 
 -- | Convert an amount to the commodity of its assigned price, if any.  Notes:
@@ -211,16 +210,8 @@ isZeroAmount a --  a==missingamt = False
                | otherwise     = (null . filter (`elem` digits) . showAmountWithoutPriceOrCommodity) a
 
 -- | Is this amount "really" zero, regardless of the display precision ?
--- Since we are using floating point, for now just test to some high precision.
 isReallyZeroAmount :: Amount -> Bool
-isReallyZeroAmount Amount{aquantity=q} = iszero q
-  where
-   iszero =
-#ifdef DOUBLE
-    null . filter (`elem` digits) . printf ("%."++show zeroprecision++"f") where zeroprecision = 8
-#else
-    (==0)
-#endif
+isReallyZeroAmount Amount{aquantity=q} = q == 0
 
 -- | Get the string representation of an amount, based on its commodity's
 -- display settings except using the specified precision.
@@ -270,14 +261,14 @@ showAmountHelper :: Bool -> Amount -> String
 showAmountHelper _ Amount{acommodity="AUTO"} = ""
 showAmountHelper showzerocommodity a@(Amount{acommodity=c, aprice=p, astyle=AmountStyle{..}}) =
     case ascommodityside of
-      L -> printf "%s%s%s%s" c' space quantity' price
-      R -> printf "%s%s%s%s" quantity' space c' price
+      L -> printf "%s%s%s%s" (T.unpack c') space quantity' price
+      R -> printf "%s%s%s%s" quantity' space (T.unpack c') price
     where
       quantity = showamountquantity a
       displayingzero = null $ filter (`elem` digits) $ quantity
       (quantity',c') | displayingzero && not showzerocommodity = ("0","")
                      | otherwise = (quantity, quoteCommoditySymbolIfNeeded c)
-      space = if (not (null c') && ascommodityspaced) then " " else "" :: String
+      space = if (not (T.null c') && ascommodityspaced) then " " else "" :: String
       price = showPrice p
 
 -- | Like showAmount, but show a zero amount's commodity if it has one.
@@ -292,15 +283,9 @@ showamountquantity Amount{aquantity=q, astyle=AmountStyle{asprecision=p, asdecim
     where
       -- isint n = fromIntegral (round n) == n
       qstr -- p == maxprecision && isint q = printf "%d" (round q::Integer)
-#ifdef DOUBLE
-        | p == maxprecisionwithpoint = printf "%f" q
-        | p == maxprecision          = chopdotzero $ printf "%f" q
-        | otherwise                  = printf ("%."++show p++"f") q
-#else
         | p == maxprecisionwithpoint = show q
         | p == maxprecision          = chopdotzero $ show q
         | otherwise                  = show $ roundTo (fromIntegral p) q
-#endif
 
 -- | Replace a number string's decimal point with the specified character,
 -- and add the specified digit group separators. The last digit group will
@@ -340,7 +325,7 @@ maxprecisionwithpoint = 999999
 
 -- like journalCanonicaliseAmounts
 -- | Canonicalise an amount's display style using the provided commodity style map.
-canonicaliseAmount :: M.Map Commodity AmountStyle -> Amount -> Amount
+canonicaliseAmount :: M.Map CommoditySymbol AmountStyle -> Amount -> Amount
 canonicaliseAmount styles a@Amount{acommodity=c, astyle=s} = a{astyle=s'}
     where
       s' = findWithDefault s c styles
@@ -449,9 +434,9 @@ sumSimilarAmountsUsingFirstPrice :: [Amount] -> Amount
 sumSimilarAmountsUsingFirstPrice [] = nullamt
 sumSimilarAmountsUsingFirstPrice as = (sum as){aprice=aprice $ head as}
 
--- | Sum same-commodity amounts. If there were different prices, set
--- the price to a special marker indicating "various". Only used as a
--- rendering helper.
+-- -- | Sum same-commodity amounts. If there were different prices, set
+-- -- the price to a special marker indicating "various". Only used as a
+-- -- rendering helper.
 -- sumSimilarAmountsNotingPriceDifference :: [Amount] -> Amount
 -- sumSimilarAmountsNotingPriceDifference [] = nullamt
 -- sumSimilarAmountsNotingPriceDifference as = undefined
@@ -468,7 +453,7 @@ filterMixedAmount p (Mixed as) = Mixed $ filter p as
 -- with the specified commodity and the quantity of that commodity
 -- found in the original. NB if Amount's quantity is zero it will be
 -- discarded next time the MixedAmount gets normalised.
-filterMixedAmountByCommodity :: Commodity -> MixedAmount -> MixedAmount
+filterMixedAmountByCommodity :: CommoditySymbol -> MixedAmount -> MixedAmount
 filterMixedAmountByCommodity c (Mixed as) = Mixed as'
   where
     as' = case filter ((==c) . acommodity) as of
@@ -580,7 +565,7 @@ showMixedAmountOneLineWithoutPrice m = concat $ intersperse ", " $ map showAmoun
       stripPrices (Mixed as) = Mixed $ map stripprice as where stripprice a = a{aprice=NoPrice}
 
 -- | Canonicalise a mixed amount's display styles using the provided commodity style map.
-canonicaliseMixedAmount :: M.Map Commodity AmountStyle -> MixedAmount -> MixedAmount
+canonicaliseMixedAmount :: M.Map CommoditySymbol AmountStyle -> MixedAmount -> MixedAmount
 canonicaliseMixedAmount styles (Mixed as) = Mixed $ map (canonicaliseAmount styles) as
 
 -------------------------------------------------------------------------------
