@@ -1,6 +1,8 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE NoMonoLocalBinds #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE OverloadedStrings #-}
 {-|
 
 Date parsing and utilities for hledger.
@@ -19,6 +21,8 @@ with both ends unspecified matches all dates.)
 An 'Interval' is ledger's \"reporting interval\" - weekly, monthly,
 quarterly, etc.
 
+'Period' will probably replace DateSpan in due course.
+
 -}
 
 -- XXX fromGregorian silently clips bad dates, use fromGregorianValid instead ?
@@ -38,7 +42,6 @@ module Hledger.Data.Dates (
   prevday,
   parsePeriodExpr,
   nulldatespan,
-  tests_Hledger_Data_Dates,
   failIfInvalidYear,
   failIfInvalidMonth,
   failIfInvalidDay,
@@ -69,6 +72,8 @@ import Prelude.Compat
 import Control.Monad
 import Data.List.Compat
 import Data.Maybe
+import Data.Text (Text)
+import qualified Data.Text as T
 #if MIN_VERSION_time(1,5,0)
 import Data.Time.Format hiding (months)
 #else
@@ -77,82 +82,30 @@ import System.Locale (TimeLocale, defaultTimeLocale)
 #endif
 import Data.Time.Calendar
 import Data.Time.Calendar.OrdinalDate
-import Data.Time.Calendar.WeekDate
 import Data.Time.Clock
 import Data.Time.LocalTime
 import Safe (headMay, lastMay, readMay)
-import Test.HUnit
-import Text.Parsec
+import Text.Megaparsec
+import Text.Megaparsec.Text
 import Text.Printf
 
 import Hledger.Data.Types
+import Hledger.Data.Period
 import Hledger.Utils
 
 
 -- Help ppShow parse and line-wrap DateSpans better in debug output.
 instance Show DateSpan where
-    show (DateSpan s1 s2) = "DateSpan \"" ++ show s1 ++ "\" \"" ++ show s2 ++ "\""
+    show s = "DateSpan " ++ showDateSpan s
+    -- show s = "DateSpan \"" ++ showDateSpan s ++ "\"" -- quotes to help pretty-show
 
 showDate :: Day -> String
 showDate = formatTime defaultTimeLocale "%0C%y/%m/%d"
 
--- XXX review for more boundary crossing issues
 -- | Render a datespan as a display string, abbreviating into a
 -- compact form if possible.
-showDateSpan ds@(DateSpan (Just from) (Just to)) =
-  case (toGregorian from, toGregorian to) of
-    -- special cases we can abbreviate:
-    -- a year, YYYY
-    ((fy,1,1), (ty,1,1))   | fy+1==ty           -> formatTime defaultTimeLocale "%0C%y" from
-    -- a half, YYYYhN
-    ((fy,1,1), (ty,7,1))   | fy==ty             -> formatTime defaultTimeLocale "%0C%yh1" from
-    ((fy,7,1), (ty,1,1))   | fy+1==ty           -> formatTime defaultTimeLocale "%0C%yh2" from
-    -- a quarter, YYYYqN
-    ((fy,1,1), (ty,4,1))   | fy==ty             -> formatTime defaultTimeLocale "%0C%yq1" from
-    ((fy,4,1), (ty,7,1))   | fy==ty             -> formatTime defaultTimeLocale "%0C%yq2" from
-    ((fy,7,1), (ty,10,1))  | fy==ty             -> formatTime defaultTimeLocale "%0C%yq3" from
-    ((fy,10,1), (ty,1,1))  | fy+1==ty           -> formatTime defaultTimeLocale "%0C%yq4" from
-    -- a month, YYYY/MM
-    ((fy,fm,1), (ty,tm,1)) | fy==ty && fm+1==tm -> formatTime defaultTimeLocale "%0C%y/%m" from
-    ((fy,12,1), (ty,1,1))  | fy+1==ty           -> formatTime defaultTimeLocale "%0C%y/%m" from
-    -- a week (two successive mondays),
-    -- YYYYwN ("week N of year YYYY")
-    -- _ | let ((fy,fw,fd), (ty,tw,td)) = (toWeekDate from, toWeekDate to) in fy==ty && fw+1==tw && fd==1 && td==1
-    --                                             -> formatTime defaultTimeLocale "%0f%gw%V" from
-    -- YYYY/MM/DDwN ("week N, starting on YYYY/MM/DD")
-    _ | let ((fy,fw,fd), (ty,tw,td)) = (toWeekDate from, toWeekDate (addDays (-1) to)) in fy==ty && fw==tw && fd==1 && td==7
-                                                -> formatTime defaultTimeLocale "%0C%y/%m/%dw%V" from
-    -- a day, YYYY/MM/DDd (d suffix is to distinguish from a regular date in register)
-    ((fy,fm,fd), (ty,tm,td)) | fy==ty && fm==tm && fd+1==td -> formatTime defaultTimeLocale "%0C%y/%m/%dd" from
-    -- crossing a year boundary
-    ((fy,fm,fd), (ty,tm,td)) | fy+1==ty && fm==12 && tm==1 && fd==31 && td==1 -> formatTime defaultTimeLocale "%0C%y/%m/%dd" from
-    -- crossing a month boundary XXX wrongly shows LEAPYEAR/2/28-LEAPYEAR/3/1 as LEAPYEAR/2/28
-    ((fy,fm,fd), (ty,tm,td)) | fy==ty && fm+1==tm && fd `elem` fromMaybe [] (lookup fm lastdayofmonth) && td==1 -> formatTime defaultTimeLocale "%0C%y/%m/%dd" from
-    -- otherwise, YYYY/MM/DD-YYYY/MM/DD
-    _                                           -> showDateSpan' ds
-  where lastdayofmonth = [(1,[31])
-                         ,(2,[28,29])
-                         ,(3,[31])
-                         ,(4,[30])
-                         ,(5,[31])
-                         ,(6,[30])
-                         ,(7,[31])
-                         ,(8,[31])
-                         ,(9,[30])
-                         ,(10,[31])
-                         ,(11,[30])
-                         ,(12,[31])
-                         ]
-
-showDateSpan ds = showDateSpan' ds
-
--- | Render a datespan as a display string.
-showDateSpan' (DateSpan from to) =
-  concat
-    [maybe "" showDate from
-    ,"-"
-    ,maybe "" (showDate . prevday) to
-    ]
+showDateSpan :: DateSpan -> String
+showDateSpan = showPeriod . dateSpanAsPeriod
 
 -- | Get the current local date.
 getCurrentDay :: IO Day
@@ -189,6 +142,32 @@ spansSpan spans = DateSpan (maybe Nothing spanStart $ headMay spans) (maybe Noth
 
 -- | Split a DateSpan into one or more consecutive whole spans of the specified length which enclose it.
 -- If no interval is specified, the original span is returned.
+--
+-- ==== Examples:
+-- >>> let t i d1 d2 = splitSpan i $ mkdatespan d1 d2
+-- >>> t NoInterval "2008/01/01" "2009/01/01"
+-- [DateSpan 2008]
+-- >>> t (Quarters 1) "2008/01/01" "2009/01/01"
+-- [DateSpan 2008q1,DateSpan 2008q2,DateSpan 2008q3,DateSpan 2008q4]
+-- >>> splitSpan (Quarters 1) nulldatespan
+-- [DateSpan -]
+-- >>> t (Days 1) "2008/01/01" "2008/01/01"  -- an empty datespan
+-- [DateSpan 2008/01/01-2007/12/31]
+-- >>> t (Quarters 1) "2008/01/01" "2008/01/01"
+-- [DateSpan 2008/01/01-2007/12/31]
+-- >>> t (Months 1) "2008/01/01" "2008/04/01"
+-- [DateSpan 2008/01,DateSpan 2008/02,DateSpan 2008/03]
+-- >>> t (Months 2) "2008/01/01" "2008/04/01"
+-- [DateSpan 2008/01/01-2008/02/29,DateSpan 2008/03/01-2008/04/30]
+-- >>> t (Weeks 1) "2008/01/01" "2008/01/15"
+-- [DateSpan 2007/12/31w01,DateSpan 2008/01/07w02,DateSpan 2008/01/14w03]
+-- >>> t (Weeks 2) "2008/01/01" "2008/01/15"
+-- [DateSpan 2007/12/31-2008/01/13,DateSpan 2008/01/14-2008/01/27]
+-- >>> t (DayOfMonth 2) "2008/01/01" "2008/04/01"
+-- [DateSpan 2008/01/02-2008/02/01,DateSpan 2008/02/02-2008/03/01,DateSpan 2008/03/02-2008/04/01]
+-- >>> t (DayOfWeek 2) "2011/01/01" "2011/01/15"
+-- [DateSpan 2011/01/04-2011/01/10,DateSpan 2011/01/11-2011/01/17]
+--
 splitSpan :: Interval -> DateSpan -> [DateSpan]
 splitSpan _ (DateSpan Nothing Nothing) = [DateSpan Nothing Nothing]
 splitSpan NoInterval     s = [s]
@@ -271,10 +250,10 @@ earliest (Just d1) (Just d2) = Just $ min d1 d2
 
 -- | Parse a period expression to an Interval and overall DateSpan using
 -- the provided reference date, or return a parse error.
-parsePeriodExpr :: Day -> String -> Either ParseError (Interval, DateSpan)
+parsePeriodExpr :: Day -> Text -> Either (ParseError Char Dec) (Interval, DateSpan)
 parsePeriodExpr refdate = parsewith (periodexpr refdate <* eof)
 
-maybePeriod :: Day -> String -> Maybe (Interval,DateSpan)
+maybePeriod :: Day -> Text -> Maybe (Interval,DateSpan)
 maybePeriod refdate = either (const Nothing) Just . parsePeriodExpr refdate
 
 -- | Show a DateSpan as a human-readable pseudo-period-expression string.
@@ -327,22 +306,93 @@ spanFromSmartDate refdate sdate = DateSpan (Just b) (Just e)
 
 -- | Convert a smart date string to an explicit yyyy\/mm\/dd string using
 -- the provided reference date, or raise an error.
-fixSmartDateStr :: Day -> String -> String
+fixSmartDateStr :: Day -> Text -> String
 fixSmartDateStr d s = either
                        (\e->error' $ printf "could not parse date %s %s" (show s) (show e))
                        id
-                       $ fixSmartDateStrEither d s
+                       $ (fixSmartDateStrEither d s :: Either (ParseError Char Dec) String)
 
 -- | A safe version of fixSmartDateStr.
-fixSmartDateStrEither :: Day -> String -> Either ParseError String
+fixSmartDateStrEither :: Day -> Text -> Either (ParseError Char Dec) String
 fixSmartDateStrEither d = either Left (Right . showDate) . fixSmartDateStrEither' d
 
-fixSmartDateStrEither' :: Day -> String -> Either ParseError Day
-fixSmartDateStrEither' d s = case parsewith smartdateonly (lowercase s) of
+fixSmartDateStrEither' :: Day -> Text -> Either (ParseError Char Dec) Day
+fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
                                Right sd -> Right $ fixSmartDate d sd
                                Left e -> Left e
 
 -- | Convert a SmartDate to an absolute date using the provided reference date.
+--
+-- ==== Examples:
+-- >>> let t = fixSmartDateStr (parsedate "2008/11/26")
+-- >>> t "0000-01-01"
+-- "0000/01/01"
+-- >>> t "1999-12-02"
+-- "1999/12/02"
+-- >>> t "1999.12.02"
+-- "1999/12/02"
+-- >>> t "1999/3/2"
+-- "1999/03/02"
+-- >>> t "19990302"
+-- "1999/03/02"
+-- >>> t "2008/2"
+-- "2008/02/01"
+-- >>> t "0020/2"
+-- "0020/02/01"
+-- >>> t "1000"
+-- "1000/01/01"
+-- >>> t "4/2"
+-- "2008/04/02"
+-- >>> t "2"
+-- "2008/11/02"
+-- >>> t "January"
+-- "2008/01/01"
+-- >>> t "feb"
+-- "2008/02/01"
+-- >>> t "today"
+-- "2008/11/26"
+-- >>> t "yesterday"
+-- "2008/11/25"
+-- >>> t "tomorrow"
+-- "2008/11/27"
+-- >>> t "this day"
+-- "2008/11/26"
+-- >>> t "last day"
+-- "2008/11/25"
+-- >>> t "next day"
+-- "2008/11/27"
+-- >>> t "this week"  -- last monday
+-- "2008/11/24"
+-- >>> t "last week"  -- previous monday
+-- "2008/11/17"
+-- >>> t "next week"  -- next monday
+-- "2008/12/01"
+-- >>> t "this month"
+-- "2008/11/01"
+-- >>> t "last month"
+-- "2008/10/01"
+-- >>> t "next month"
+-- "2008/12/01"
+-- >>> t "this quarter"
+-- "2008/10/01"
+-- >>> t "last quarter"
+-- "2008/07/01"
+-- >>> t "next quarter"
+-- "2009/01/01"
+-- >>> t "this year"
+-- "2008/01/01"
+-- >>> t "last year"
+-- "2007/01/01"
+-- >>> t "next year"
+-- "2009/01/01"
+--
+-- t "last wed"
+-- "2008/11/19"
+-- t "next friday"
+-- "2008/11/28"
+-- t "next january"
+-- "2009/01/01"
+--
 fixSmartDate :: Day -> SmartDate -> Day
 fixSmartDate refdate sdate = fix sdate
     where
@@ -448,15 +498,34 @@ parsedateM s = firstJust [
 -- parsedatetime s = fromMaybe (error' $ "could not parse timestamp \"" ++ s ++ "\"")
 --                             (parsedatetimeM s)
 
--- | Parse a date string to a time type, or raise an error.
+-- | Parse a YYYY-MM-DD or YYYY/MM/DD date string to a Day, or raise an error. For testing/debugging.
+--
+-- >>> parsedate "2008/02/03"
+-- 2008-02-03
 parsedate :: String -> Day
 parsedate s =  fromMaybe (error' $ "could not parse date \"" ++ s ++ "\"")
                          (parsedateM s)
+-- doctests I haven't been able to make compatible with both GHC 7 and 8
+-- -- >>> parsedate "2008/02/03/"
+-- -- *** Exception: could not parse date "2008/02/03/"
+-- #if MIN_VERSION_base(4,9,0)
+-- -- ...
+-- #endif
+-- #if MIN_VERSION_time(1,6,0)
+-- -- >>> parsedate "2008/02/30"  -- with time >= 1.6, invalid dates are rejected
+-- -- *** Exception: could not parse date "2008/02/30"
+-- #if MIN_VERSION_base(4,9,0)
+-- -- ...
+-- #endif
+-- #else
+-- -- >>> parsedate "2008/02/30"  -- with time < 1.6, they are silently adjusted
+-- -- 2008-02-29
+-- #endif
 
 -- | Parse a time string to a time type using the provided pattern, or
 -- return the default.
-parsetimewith :: ParseTime t => String -> String -> t -> t
-parsetimewith pat s def = fromMaybe def $ parsetime defaultTimeLocale pat s
+_parsetimewith :: ParseTime t => String -> String -> t -> t
+_parsetimewith pat s def = fromMaybe def $ parsetime defaultTimeLocale pat s
 
 {-|
 Parse a date in any of the formats allowed in ledger's period expressions,
@@ -474,22 +543,23 @@ and maybe some others:
 Returns a SmartDate, to be converted to a full date later (see fixSmartDate).
 Assumes any text in the parse stream has been lowercased.
 -}
-smartdate :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+smartdate :: Parser SmartDate
 smartdate = do
   -- XXX maybe obscures date errors ? see ledgerdate
   (y,m,d) <- choice' [yyyymmdd, ymd, ym, md, y, d, month, mon, today, yesterday, tomorrow, lastthisnextthing]
   return (y,m,d)
 
 -- | Like smartdate, but there must be nothing other than whitespace after the date.
-smartdateonly :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+smartdateonly :: Parser SmartDate
 smartdateonly = do
   d <- smartdate
   many spacenonewline
   eof
   return d
 
+datesepchars :: [Char]
 datesepchars = "/-."
-datesepchar :: Stream [Char] m Char => ParsecT [Char] st m Char
+datesepchar :: TextParser m Char
 datesepchar = oneOf datesepchars
 
 validYear, validMonth, validDay :: String -> Bool
@@ -502,54 +572,54 @@ failIfInvalidYear s  = unless (validYear s)  $ fail $ "bad year number: " ++ s
 failIfInvalidMonth s = unless (validMonth s) $ fail $ "bad month number: " ++ s
 failIfInvalidDay s   = unless (validDay s)   $ fail $ "bad day number: " ++ s
 
-yyyymmdd :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+yyyymmdd :: Parser SmartDate
 yyyymmdd = do
-  y <- count 4 digit
-  m <- count 2 digit
+  y <- count 4 digitChar
+  m <- count 2 digitChar
   failIfInvalidMonth m
-  d <- count 2 digit
+  d <- count 2 digitChar
   failIfInvalidDay d
   return (y,m,d)
 
-ymd :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+ymd :: Parser SmartDate
 ymd = do
-  y <- many1 digit
+  y <- some digitChar
   failIfInvalidYear y
   sep <- datesepchar
-  m <- many1 digit
+  m <- some digitChar
   failIfInvalidMonth m
   char sep
-  d <- many1 digit
+  d <- some digitChar
   failIfInvalidDay d
   return $ (y,m,d)
 
-ym :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+ym :: Parser SmartDate
 ym = do
-  y <- many1 digit
+  y <- some digitChar
   failIfInvalidYear y
   datesepchar
-  m <- many1 digit
+  m <- some digitChar
   failIfInvalidMonth m
   return (y,m,"")
 
-y :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+y :: Parser SmartDate
 y = do
-  y <- many1 digit
+  y <- some digitChar
   failIfInvalidYear y
   return (y,"","")
 
-d :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+d :: Parser SmartDate
 d = do
-  d <- many1 digit
+  d <- some digitChar
   failIfInvalidDay d
   return ("","",d)
 
-md :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+md :: Parser SmartDate
 md = do
-  m <- many1 digit
+  m <- some digitChar
   failIfInvalidMonth m
   datesepchar
-  d <- many1 digit
+  d <- some digitChar
   failIfInvalidDay d
   return ("",m,d)
 
@@ -562,24 +632,24 @@ monthabbrevs   = ["jan","feb","mar","apr","may","jun","jul","aug","sep","oct","n
 monthIndex s = maybe 0 (+1) $ lowercase s `elemIndex` months
 monIndex s   = maybe 0 (+1) $ lowercase s `elemIndex` monthabbrevs
 
-month :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+month :: Parser SmartDate
 month = do
   m <- choice $ map (try . string) months
   let i = monthIndex m
   return ("",show i,"")
 
-mon :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+mon :: Parser SmartDate
 mon = do
   m <- choice $ map (try . string) monthabbrevs
   let i = monIndex m
   return ("",show i,"")
 
-today,yesterday,tomorrow :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+today,yesterday,tomorrow :: Parser SmartDate
 today     = string "today"     >> return ("","","today")
 yesterday = string "yesterday" >> return ("","","yesterday")
 tomorrow  = string "tomorrow"  >> return ("","","tomorrow")
 
-lastthisnextthing :: Stream [Char] m Char => ParsecT [Char] st m SmartDate
+lastthisnextthing :: Parser SmartDate
 lastthisnextthing = do
   r <- choice [
         string "last"
@@ -599,7 +669,19 @@ lastthisnextthing = do
 
   return ("",r,p)
 
-periodexpr :: Stream [Char] m Char => Day -> ParsecT [Char] st m (Interval, DateSpan)
+-- |
+-- >>> let p = parsewith (periodexpr (parsedate "2008/11/26")) :: T.Text -> Either (ParseError Char Dec) (Interval, DateSpan)
+-- >>> p "from aug to oct"
+-- Right (NoInterval,DateSpan 2008/08/01-2008/09/30)
+-- >>> p "aug to oct"
+-- Right (NoInterval,DateSpan 2008/08/01-2008/09/30)
+-- >>> p "every 3 days in aug"
+-- Right (Days 3,DateSpan 2008/08)
+-- >>> p "daily from aug"
+-- Right (Days 1,DateSpan 2008/08/01-)
+-- >>> p "every week to 2009"
+-- Right (Weeks 1,DateSpan -2008/12/31)
+periodexpr :: Day -> Parser (Interval, DateSpan)
 periodexpr rdate = choice $ map try [
                     intervalanddateperiodexpr rdate,
                     intervalperiodexpr,
@@ -607,7 +689,7 @@ periodexpr rdate = choice $ map try [
                     (return (NoInterval,DateSpan Nothing Nothing))
                    ]
 
-intervalanddateperiodexpr :: Stream [Char] m Char => Day -> ParsecT [Char] st m (Interval, DateSpan)
+intervalanddateperiodexpr :: Day -> Parser (Interval, DateSpan)
 intervalanddateperiodexpr rdate = do
   many spacenonewline
   i <- reportinginterval
@@ -615,20 +697,20 @@ intervalanddateperiodexpr rdate = do
   s <- periodexprdatespan rdate
   return (i,s)
 
-intervalperiodexpr :: Stream [Char] m Char => ParsecT [Char] st m (Interval, DateSpan)
+intervalperiodexpr :: Parser (Interval, DateSpan)
 intervalperiodexpr = do
   many spacenonewline
   i <- reportinginterval
   return (i, DateSpan Nothing Nothing)
 
-dateperiodexpr :: Stream [Char] m Char => Day -> ParsecT [Char] st m (Interval, DateSpan)
+dateperiodexpr :: Day -> Parser (Interval, DateSpan)
 dateperiodexpr rdate = do
   many spacenonewline
   s <- periodexprdatespan rdate
   return (NoInterval, s)
 
 -- Parse a reporting interval.
-reportinginterval :: Stream [Char] m Char => ParsecT [Char] st m Interval
+reportinginterval :: Parser Interval
 reportinginterval = choice' [
                        tryinterval "day"     "daily"     Days,
                        tryinterval "week"    "weekly"    Weeks,
@@ -641,7 +723,7 @@ reportinginterval = choice' [
                           return $ Months 2,
                        do string "every"
                           many spacenonewline
-                          n <- fmap read $ many1 digit
+                          n <- fmap read $ some digitChar
                           thsuffix
                           many spacenonewline
                           string "day"
@@ -652,7 +734,7 @@ reportinginterval = choice' [
                           return $ DayOfWeek n,
                        do string "every"
                           many spacenonewline
-                          n <- fmap read $ many1 digit
+                          n <- fmap read $ some digitChar
                           thsuffix
                           many spacenonewline
                           string "day"
@@ -668,7 +750,7 @@ reportinginterval = choice' [
       thsuffix = choice' $ map string ["st","nd","rd","th"]
 
       -- Parse any of several variants of a basic interval, eg "daily", "every day", "every N days".
-      tryinterval :: Stream [Char] m Char => String -> String -> (Int -> Interval) -> ParsecT [Char] st m Interval
+      tryinterval :: String -> String -> (Int -> Interval) -> Parser Interval
       tryinterval singular compact intcons =
           choice' [
            do string compact
@@ -679,14 +761,14 @@ reportinginterval = choice' [
               return $ intcons 1,
            do string "every"
               many spacenonewline
-              n <- fmap read $ many1 digit
+              n <- fmap read $ some digitChar
               many spacenonewline
               string plural
               return $ intcons n
            ]
           where plural = singular ++ "s"
 
-periodexprdatespan :: Stream [Char] m Char => Day -> ParsecT [Char] st m DateSpan
+periodexprdatespan :: Day -> Parser DateSpan
 periodexprdatespan rdate = choice $ map try [
                             doubledatespan rdate,
                             fromdatespan rdate,
@@ -694,7 +776,7 @@ periodexprdatespan rdate = choice $ map try [
                             justdatespan rdate
                            ]
 
-doubledatespan :: Stream [Char] m Char => Day -> ParsecT [Char] st m DateSpan
+doubledatespan :: Day -> Parser DateSpan
 doubledatespan rdate = do
   optional (string "from" >> many spacenonewline)
   b <- smartdate
@@ -703,7 +785,7 @@ doubledatespan rdate = do
   e <- smartdate
   return $ DateSpan (Just $ fixSmartDate rdate b) (Just $ fixSmartDate rdate e)
 
-fromdatespan :: Stream [Char] m Char => Day -> ParsecT [Char] st m DateSpan
+fromdatespan :: Day -> Parser DateSpan
 fromdatespan rdate = do
   b <- choice [
     do
@@ -717,13 +799,13 @@ fromdatespan rdate = do
     ]
   return $ DateSpan (Just $ fixSmartDate rdate b) Nothing
 
-todatespan :: Stream [Char] m Char => Day -> ParsecT [Char] st m DateSpan
+todatespan :: Day -> Parser DateSpan
 todatespan rdate = do
   choice [string "to", string "-"] >> many spacenonewline
   e <- smartdate
   return $ DateSpan Nothing (Just $ fixSmartDate rdate e)
 
-justdatespan :: Stream [Char] m Char => Day -> ParsecT [Char] st m DateSpan
+justdatespan :: Day -> Parser DateSpan
 justdatespan rdate = do
   optional (string "in" >> many spacenonewline)
   d <- smartdate
@@ -738,103 +820,4 @@ nulldatespan :: DateSpan
 nulldatespan = DateSpan Nothing Nothing
 
 nulldate :: Day
-nulldate = parsedate "0000/00/00"
-
-tests_Hledger_Data_Dates = TestList
- [
-
-   "parsedate" ~: do
-    let date1 = parsedate "2008/11/26"
-    parsedate "2008/02/03" `is` parsetimewith "%Y/%m/%d" "2008/02/03" date1
-    parsedate "2008-02-03" `is` parsetimewith "%Y/%m/%d" "2008/02/03" date1
-
-  ,"period expressions" ~: do
-    let todaysdate = parsedate "2008/11/26"
-    let str `gives` result = show (parsewith (periodexpr todaysdate) str) `is` ("Right " ++ result)
-    "from aug to oct"           `gives` "(NoInterval,DateSpan \"Just 2008-08-01\" \"Just 2008-10-01\")"
-    "aug to oct"                `gives` "(NoInterval,DateSpan \"Just 2008-08-01\" \"Just 2008-10-01\")"
-    "every 3 days in aug"       `gives` "(Days 3,DateSpan \"Just 2008-08-01\" \"Just 2008-09-01\")"
-    "daily from aug"            `gives` "(Days 1,DateSpan \"Just 2008-08-01\" \"Nothing\")"
-    "every week to 2009"        `gives` "(Weeks 1,DateSpan \"Nothing\" \"Just 2009-01-01\")"
-
-  ,"splitSpan" ~: do
-    let gives (interval, span) = (splitSpan interval span `is`)
-    (NoInterval,mkdatespan "2008/01/01" "2009/01/01") `gives`
-     [mkdatespan "2008/01/01" "2009/01/01"]
-    (Quarters 1,mkdatespan "2008/01/01" "2009/01/01") `gives`
-     [mkdatespan "2008/01/01" "2008/04/01"
-     ,mkdatespan "2008/04/01" "2008/07/01"
-     ,mkdatespan "2008/07/01" "2008/10/01"
-     ,mkdatespan "2008/10/01" "2009/01/01"
-     ]
-    (Quarters 1,nulldatespan) `gives`
-     [nulldatespan]
-    (Days 1,mkdatespan "2008/01/01" "2008/01/01") `gives`
-     [mkdatespan "2008/01/01" "2008/01/01"]
-    (Quarters 1,mkdatespan "2008/01/01" "2008/01/01") `gives`
-     [mkdatespan "2008/01/01" "2008/01/01"]
-    (Months 1,mkdatespan "2008/01/01" "2008/04/01") `gives`
-     [mkdatespan "2008/01/01" "2008/02/01"
-     ,mkdatespan "2008/02/01" "2008/03/01"
-     ,mkdatespan "2008/03/01" "2008/04/01"
-     ]
-    (Months 2,mkdatespan "2008/01/01" "2008/04/01") `gives`
-     [mkdatespan "2008/01/01" "2008/03/01"
-     ,mkdatespan "2008/03/01" "2008/05/01"
-     ]
-    (Weeks 1,mkdatespan "2008/01/01" "2008/01/15") `gives`
-     [mkdatespan "2007/12/31" "2008/01/07"
-     ,mkdatespan "2008/01/07" "2008/01/14"
-     ,mkdatespan "2008/01/14" "2008/01/21"
-     ]
-    (Weeks 2,mkdatespan "2008/01/01" "2008/01/15") `gives`
-     [mkdatespan "2007/12/31" "2008/01/14"
-     ,mkdatespan "2008/01/14" "2008/01/28"
-     ]
-    (DayOfMonth 2,mkdatespan "2008/01/01" "2008/04/01") `gives`
-     [mkdatespan "2008/01/02" "2008/02/02"
-     ,mkdatespan "2008/02/02" "2008/03/02"
-     ,mkdatespan "2008/03/02" "2008/04/02"
-     ]
-    (DayOfWeek 2,mkdatespan "2011/01/01" "2011/01/15") `gives`
-     [mkdatespan "2011/01/04" "2011/01/11"
-     ,mkdatespan "2011/01/11" "2011/01/18"
-     ]
-
-  ,"fixSmartDateStr" ~: do
-    let gives = is . fixSmartDateStr (parsedate "2008/11/26")
-    "0000-01-01"   `gives` "0000/01/01"
-    "1999-12-02"   `gives` "1999/12/02"
-    "1999.12.02"   `gives` "1999/12/02"
-    "1999/3/2"     `gives` "1999/03/02"
-    "19990302"     `gives` "1999/03/02"
-    "2008/2"       `gives` "2008/02/01"
-    "0020/2"       `gives` "0020/02/01"
-    "1000"         `gives` "1000/01/01"
-    "4/2"          `gives` "2008/04/02"
-    "2"            `gives` "2008/11/02"
-    "January"      `gives` "2008/01/01"
-    "feb"          `gives` "2008/02/01"
-    "today"        `gives` "2008/11/26"
-    "yesterday"    `gives` "2008/11/25"
-    "tomorrow"     `gives` "2008/11/27"
-    "this day"     `gives` "2008/11/26"
-    "last day"     `gives` "2008/11/25"
-    "next day"     `gives` "2008/11/27"
-    "this week"    `gives` "2008/11/24" -- last monday
-    "last week"    `gives` "2008/11/17" -- previous monday
-    "next week"    `gives` "2008/12/01" -- next monday
-    "this month"   `gives` "2008/11/01"
-    "last month"   `gives` "2008/10/01"
-    "next month"   `gives` "2008/12/01"
-    "this quarter" `gives` "2008/10/01"
-    "last quarter" `gives` "2008/07/01"
-    "next quarter" `gives` "2009/01/01"
-    "this year"    `gives` "2008/01/01"
-    "last year"    `gives` "2007/01/01"
-    "next year"    `gives` "2009/01/01"
---     "last wed"     `gives` "2008/11/19"
---     "next friday"  `gives` "2008/11/28"
---     "next january" `gives` "2009/01/01"
-
- ]
+nulldate = fromGregorian 0 1 1
