@@ -1,4 +1,4 @@
--- {-# LANGUAGE CPP #-}
+{-# LANGUAGE StandaloneDeriving, OverloadedStrings #-}
 {-|
 
 A 'Journal' is a set of transactions, plus optional related data.  This is
@@ -12,11 +12,11 @@ module Hledger.Data.Journal (
   addMarketPrice,
   addModifierTransaction,
   addPeriodicTransaction,
-  addTimeLogEntry,
   addTransaction,
   journalApplyAliases,
   journalBalanceTransactions,
-  journalCanonicaliseAmounts,
+  journalApplyCommodityStyles,
+  commodityStylesFromAmounts,
   journalConvertAmountsToCost,
   journalFinalise,
   -- * Filtering
@@ -24,6 +24,7 @@ module Hledger.Data.Journal (
   filterJournalPostings,
   filterJournalAmounts,
   filterTransactionAmounts,
+  filterTransactionPostings,
   filterPostingAmount,
   -- * Querying
   journalAccountNames,
@@ -49,10 +50,12 @@ module Hledger.Data.Journal (
   journalEquityAccountQuery,
   journalCashAccountQuery,
   -- * Misc
-  canonicalStyles,
+  canonicalStyleFrom,
   matchpats,
-  nullctx,
   nulljournal,
+  journalCheckBalanceAssertions,
+  journalNumberAndTieTransactions,
+  journalUntieTransactions,
   -- * Tests
   samplejournal,
   tests_Hledger_Data_Journal,
@@ -62,12 +65,13 @@ import Control.Monad
 import Data.List
 -- import Data.Map (findWithDefault)
 import Data.Maybe
+import Data.Monoid
 import Data.Ord
-import Safe (headMay)
+import Data.Text (Text)
+import qualified Data.Text as T
+import Safe (headMay, headDef)
 import Data.Time.Calendar
-import Data.Time.LocalTime
 import Data.Tree
-import Safe (headDef)
 import System.Time (ClockTime(TOD))
 import Test.HUnit
 import Text.Printf
@@ -81,10 +85,14 @@ import Hledger.Data.Amount
 import Hledger.Data.Dates
 import Hledger.Data.Transaction
 import Hledger.Data.Posting
-import Hledger.Data.TimeLog
 import Hledger.Query
 
 
+-- try to make Journal ppShow-compatible
+-- instance Show ClockTime where
+--   show t = "<ClockTime>"
+-- deriving instance Show Journal
+
 instance Show Journal where
   show j
     | debugLevel < 3 = printf "Journal %s with %d transactions, %d accounts"
@@ -107,7 +115,7 @@ instance Show Journal where
               length (jperiodictxns j))
              (length accounts)
              (show accounts)
-             (show $ jcommoditystyles j)
+             (show $ jinferredcommodities j)
              -- ++ (show $ journalTransactions l)
              where accounts = filter (/= "root") $ flatten $ journalAccountNameTree j
 
@@ -116,37 +124,73 @@ instance Show Journal where
 --                      ,show (jtxns j)
 --                      ,show (jmodifiertxns j)
 --                      ,show (jperiodictxns j)
---                      ,show $ open_timelog_entries j
+--                      ,show $ jparsetimeclockentries j
 --                      ,show $ jmarketprices j
---                      ,show $ final_comment_lines j
---                      ,show $ jContext j
---                      ,show $ map fst $ files j
+--                      ,show $ jfinalcommentlines j
+--                      ,show $ jparsestate j
+--                      ,show $ map fst $ jfiles j
 --                      ]
 
-nulljournal :: Journal
-nulljournal = Journal { jmodifiertxns = []
-                      , jperiodictxns = []
-                      , jtxns = []
-                      , open_timelog_entries = []
-                      , jmarketprices = []
-                      , final_comment_lines = []
-                      , jContext = nullctx
-                      , files = []
-                      , filereadtime = TOD 0 0
-                      , jcommoditystyles = M.fromList []
-                      }
+-- The monoid instance for Journal is useful for two situations.
+-- 
+-- 1. concatenating finalised journals, eg with multiple -f options:
+-- FIRST <> SECOND. The second's list fields are appended to the
+-- first's, map fields are combined, transaction counts are summed,
+-- the parse state of the second is kept.
+-- 
+-- 2. merging a child parsed journal, eg with the include directive:
+-- CHILD <> PARENT. A parsed journal's data is in reverse order, so
+-- this gives what we want.
+--
+instance Monoid Journal where
+  mempty = nulljournal
+  mappend j1 j2 = Journal {
+     jparsedefaultyear          = jparsedefaultyear          j2
+    ,jparsedefaultcommodity     = jparsedefaultcommodity     j2
+    ,jparseparentaccounts       = jparseparentaccounts       j2
+    ,jparsealiases              = jparsealiases              j2
+    -- ,jparsetransactioncount     = jparsetransactioncount     j1 +  jparsetransactioncount     j2
+    ,jparsetimeclockentries = jparsetimeclockentries j1 <> jparsetimeclockentries j2
+    ,jaccounts                  = jaccounts                  j1 <> jaccounts                  j2
+    ,jcommodities               = jcommodities               j1 <> jcommodities               j2
+    ,jinferredcommodities       = jinferredcommodities       j1 <> jinferredcommodities       j2
+    ,jmarketprices              = jmarketprices              j1 <> jmarketprices              j2
+    ,jmodifiertxns              = jmodifiertxns              j1 <> jmodifiertxns              j2
+    ,jperiodictxns              = jperiodictxns              j1 <> jperiodictxns              j2
+    ,jtxns                      = jtxns                      j1 <> jtxns                      j2
+    ,jfinalcommentlines         = jfinalcommentlines         j2
+    ,jfiles                     = jfiles                     j1 <> jfiles                     j2
+    ,jlastreadtime              = max (jlastreadtime j1) (jlastreadtime j2)
+    }
 
-nullctx :: JournalContext
-nullctx = Ctx{ctxYear=Nothing, ctxDefaultCommodityAndStyle=Nothing, ctxAccount=[], ctxAliases=[], ctxTransactionIndex=0}
+nulljournal :: Journal
+nulljournal = Journal {
+   jparsedefaultyear          = Nothing
+  ,jparsedefaultcommodity     = Nothing
+  ,jparseparentaccounts       = []
+  ,jparsealiases              = []
+  -- ,jparsetransactioncount     = 0
+  ,jparsetimeclockentries = []
+  ,jaccounts                  = []
+  ,jcommodities               = M.fromList []
+  ,jinferredcommodities       = M.fromList []
+  ,jmarketprices              = []
+  ,jmodifiertxns              = []
+  ,jperiodictxns              = []
+  ,jtxns                      = []
+  ,jfinalcommentlines         = ""
+  ,jfiles                     = []
+  ,jlastreadtime              = TOD 0 0
+  }
 
 journalFilePath :: Journal -> FilePath
 journalFilePath = fst . mainfile
 
 journalFilePaths :: Journal -> [FilePath]
-journalFilePaths = map fst . files
+journalFilePaths = map fst . jfiles
 
-mainfile :: Journal -> (FilePath, String)
-mainfile = headDef ("", "") . files
+mainfile :: Journal -> (FilePath, Text)
+mainfile = headDef ("", "") . jfiles
 
 addTransaction :: Transaction -> Journal -> Journal
 addTransaction t j = j { jtxns = t : jtxns j }
@@ -160,9 +204,6 @@ addPeriodicTransaction pt j = j { jperiodictxns = pt : jperiodictxns j }
 addMarketPrice :: MarketPrice -> Journal -> Journal
 addMarketPrice h j = j { jmarketprices = h : jmarketprices j }
 
-addTimeLogEntry :: TimeLogEntry -> Journal -> Journal
-addTimeLogEntry tle j = j { open_timelog_entries = tle : open_timelog_entries j }
-
 -- | Get the transaction with this index (its 1-based position in the input stream), if any.
 journalTransactionAt :: Journal -> Integer -> Maybe Transaction
 journalTransactionAt Journal{jtxns=ts} i =
@@ -172,13 +213,13 @@ journalTransactionAt Journal{jtxns=ts} i =
 -- | Get the transaction that appeared immediately after this one in the input stream, if any.
 journalNextTransaction :: Journal -> Transaction -> Maybe Transaction
 journalNextTransaction j t = journalTransactionAt j (tindex t + 1)
-  
+
 -- | Get the transaction that appeared immediately before this one in the input stream, if any.
 journalPrevTransaction :: Journal -> Transaction -> Maybe Transaction
 journalPrevTransaction j t = journalTransactionAt j (tindex t - 1)
-  
+
 -- | Unique transaction descriptions used in this journal.
-journalDescriptions :: Journal -> [String]
+journalDescriptions :: Journal -> [Text]
 journalDescriptions = nub . sort . map tdescription . jtxns
 
 -- | All postings from this journal's transactions, in order.
@@ -258,9 +299,7 @@ filterJournalTransactions q j@Journal{jtxns=ts} = j{jtxns=filter (q `matchesTran
 -- | Keep only postings matching the query expression.
 -- This can leave unbalanced transactions.
 filterJournalPostings :: Query -> Journal -> Journal
-filterJournalPostings q j@Journal{jtxns=ts} = j{jtxns=map filtertransactionpostings ts}
-    where
-      filtertransactionpostings t@Transaction{tpostings=ps} = t{tpostings=filter (q `matchesPosting`) ps}
+filterJournalPostings q j@Journal{jtxns=ts} = j{jtxns=map (filterTransactionPostings q) ts}
 
 -- | Within each posting's amount, keep only the parts matching the query.
 -- This can leave unbalanced transactions.
@@ -276,6 +315,10 @@ filterTransactionAmounts q t@Transaction{tpostings=ps} = t{tpostings=map (filter
 filterPostingAmount :: Query -> Posting -> Posting
 filterPostingAmount q p@Posting{pamount=Mixed as} = p{pamount=Mixed $ filter (q `matchesAmount`) as}
 
+filterTransactionPostings :: Query -> Transaction -> Transaction
+filterTransactionPostings q t@Transaction{tpostings=ps} = t{tpostings=filter (q `matchesPosting`) ps}
+
+
 {-
 -------------------------------------------------------------------------------
 -- filtering V1
@@ -414,25 +457,40 @@ journalApplyAliases aliases j@Journal{jtxns=ts} =
       dotransaction t@Transaction{tpostings=ps} = t{tpostings=map doposting ps}
       doposting p@Posting{paccount=a} = p{paccount= accountNameApplyAliases aliases a}
 
--- | Do post-parse processing on a journal to make it ready for use: check
--- all transactions balance, canonicalise amount formats, close any open
--- timelog entries, maybe check balance assertions and so on.
-journalFinalise :: ClockTime -> LocalTime -> FilePath -> String -> JournalContext -> Bool -> Journal -> Either String Journal
-journalFinalise tclock tlocal path txt ctx assrt j@Journal{files=fs} = do
-  (journalBalanceTransactions $
-    journalCanonicaliseAmounts $
-    journalCloseTimeLogEntries tlocal $
-    j{ files=(path,txt):fs
-     , filereadtime=tclock
-     , jContext=ctx
-     , jtxns=reverse $ jtxns j -- NOTE: see addTransaction
-     , jmodifiertxns=reverse $ jmodifiertxns j -- NOTE: see addModifierTransaction
-     , jperiodictxns=reverse $ jperiodictxns j -- NOTE: see addPeriodicTransaction
-     , jmarketprices=reverse $ jmarketprices j -- NOTE: see addMarketPrice
-     , open_timelog_entries=reverse $ open_timelog_entries j -- NOTE: see addTimeLogEntry
-     })
+-- | Do post-parse processing on a parsed journal to make it ready for
+-- use.  Reverse parsed data to normal order, canonicalise amount
+-- formats, check/ensure that transactions are balanced, and maybe
+-- check balance assertions.
+journalFinalise :: ClockTime -> FilePath -> Text -> Bool -> ParsedJournal -> Either String Journal
+journalFinalise t path txt assrt j@Journal{jfiles=fs} = do
+  (journalNumberAndTieTransactions <$>
+    (journalBalanceTransactions $
+    journalApplyCommodityStyles $
+    j{ jfiles        = (path,txt) : reverse fs
+     , jlastreadtime = t
+     , jtxns         = reverse $ jtxns j -- NOTE: see addTransaction
+     , jmodifiertxns = reverse $ jmodifiertxns j -- NOTE: see addModifierTransaction
+     , jperiodictxns = reverse $ jperiodictxns j -- NOTE: see addPeriodicTransaction
+     , jmarketprices = reverse $ jmarketprices j -- NOTE: see addMarketPrice
+     }))
   >>= if assrt then journalCheckBalanceAssertions else return
 
+journalNumberAndTieTransactions = journalTieTransactions . journalNumberTransactions
+
+-- | Number (set the tindex field) this journal's transactions, counting upward from 1.
+journalNumberTransactions :: Journal -> Journal
+journalNumberTransactions j@Journal{jtxns=ts} = j{jtxns=map (\(i,t) -> t{tindex=i}) $ zip [1..] ts}
+
+-- | Tie the knot in all of this journal's transactions, ensuring their postings
+-- refer to them. This should be done last, after any other transaction-modifying operations.
+journalTieTransactions :: Journal -> Journal
+journalTieTransactions j@Journal{jtxns=ts} = j{jtxns=map txnTieKnot ts}
+
+-- | Untie all transaction-posting knots in this journal, so that eg
+-- recursiveSize and GHCI's :sprint can work on it.
+journalUntieTransactions :: Transaction -> Transaction
+journalUntieTransactions t@Transaction{tpostings=ps} = t{tpostings=map (\p -> p{ptransaction=Nothing}) ps}
+
 -- | Check any balance assertions in the journal and return an error
 -- message if any of them fail.
 journalCheckBalanceAssertions :: Journal -> Either String Journal
@@ -462,26 +520,47 @@ checkBalanceAssertionsForAccount ps
 checkBalanceAssertion :: ([String],MixedAmount) -> [Posting] -> ([String],MixedAmount)
 checkBalanceAssertion (errs,startbal) ps
   | null ps = (errs,startbal)
-  | isNothing assertion = (errs,startbal)
-  | -- bal' /= assertedbal  -- MixedAmount's Eq instance currently gets confused by different precisions
-    not $ isReallyZeroMixedAmount (bal - assertedbal) = (errs++[err], fullbal)
-  | otherwise = (errs,fullbal)
+  | isNothing $ pbalanceassertion p = (errs,startbal)
+  | iswrong = (errs++[err], finalfullbal)
+  | otherwise = (errs,finalfullbal)
   where
     p = last ps
-    assertion = pbalanceassertion p
-    Just assertedbal = dbg2 "assertedbal" assertion
-    assertedcomm = dbg2 "assertedcomm" $ maybe "" acommodity $ headMay $ amounts assertedbal
-    fullbal = dbg2 "fullbal" $ sum $ [dbg2 "startbal" startbal] ++ map pamount ps
-    singlebal = dbg2 "singlebal" $ filterMixedAmount (\a -> acommodity a == assertedcomm) fullbal
-    bal = singlebal -- check single-commodity balance like Ledger; maybe add == FULLBAL later
-    err = printf "Balance assertion failed for account %s on %s\n%sAfter posting:\n   %s\nexpected balance in commodity \"%s\" is %s, calculated balance was %s."
-                 (paccount p)
-                 (show $ postingDate p)
-                 (maybe "" (("In transaction:\n"++).show) $ ptransaction p)
-                 (show p)
+    Just assertedbal = pbalanceassertion p
+    assertedcomm = maybe "" acommodity $ headMay $ amounts assertedbal
+    finalfullbal = sum $ [startbal] ++ map pamount (dbg2 "ps" ps)
+    finalsinglebal = filterMixedAmount (\a -> acommodity a == assertedcomm) finalfullbal
+    actualbal = finalsinglebal -- just check the single-commodity balance, like Ledger; maybe add ==FULLBAL later
+    iswrong = dbg2 debugmsg $
+      not (isReallyZeroMixedAmount (actualbal - assertedbal))
+      -- bal' /= assertedbal  -- MixedAmount's Eq instance currently gets confused by different precisions
+      where
+        debugmsg = "assertions: on " ++ show (postingDate p) ++ " balance of " ++ show assertedcomm
+                    ++ " in " ++ T.unpack (paccount p) ++ " should be " ++ show assertedbal
+    diff = assertedbal - actualbal
+    diffplus | isNegativeMixedAmount diff == Just False = "+"
+             | otherwise = ""
+    err = printf (unlines [
+                      "balance assertion error%s",
+                      "after posting:",
+                      "%s",
+                      "balance assertion details:",
+                      "date:       %s",
+                      "account:    %s",
+                      "commodity:  %s",
+                      "calculated: %s",
+                      "asserted:   %s (difference: %s)"
+                      ])
+                 (case ptransaction p of
+                    Nothing -> ":" -- shouldn't happen
+                    Just t ->  printf " in \"%s\" (line %d, column %d):\nin transaction:\n%s" f l c (chomp $ show t) :: String
+                      where GenericSourcePos f l c = tsourcepos t)
+                 (showPostingLine p)
+                 (showDate $ postingDate p)
+                 (T.unpack $ paccount p) -- XXX pack
                  assertedcomm
+                 (showMixedAmount finalsinglebal)
                  (showMixedAmount assertedbal)
-                 (showMixedAmount singlebal)
+                 (diffplus ++ showMixedAmount diff)
 
 -- Given a sequence of postings to a single account, split it into
 -- sub-sequences consisting of ordinary postings followed by a single
@@ -496,55 +575,77 @@ splitAssertions ps
 
 -- | Fill in any missing amounts and check that all journal transactions
 -- balance, or return an error message. This is done after parsing all
--- amounts and working out the canonical commodities, since balancing
+-- amounts and applying canonical commodity styles, since balancing
 -- depends on display precision. Reports only the first error encountered.
 journalBalanceTransactions :: Journal -> Either String Journal
-journalBalanceTransactions j@Journal{jtxns=ts, jcommoditystyles=ss} =
-  case sequence $ map balance ts of Right ts' -> Right j{jtxns=map txnTieKnot ts'}
+journalBalanceTransactions j@Journal{jtxns=ts, jinferredcommodities=ss} =
+  case sequence $ map balance ts of Right ts' -> Right j{jtxns=ts'}
                                     Left e    -> Left e
       where balance = balanceTransaction (Just ss)
 
--- | Convert all the journal's posting amounts (and historical price
--- amounts, but currently not transaction price amounts) to their
--- canonical display settings. Ie, all amounts in a given commodity
--- will use (a) the display settings of the first, and (b) the
--- greatest precision, of the posting amounts in that commodity.
-journalCanonicaliseAmounts :: Journal -> Journal
-journalCanonicaliseAmounts j@Journal{jtxns=ts, jmarketprices=mps} = j''
+-- | Choose and apply a consistent display format to the posting
+-- amounts in each commodity. Each commodity's format is specified by
+-- a commodity format directive, or otherwise inferred from posting
+-- amounts as in hledger < 0.28.
+journalApplyCommodityStyles :: Journal -> Journal
+journalApplyCommodityStyles j@Journal{jtxns=ts, jmarketprices=mps} = j''
     where
+      j' = journalInferCommodityStyles j
       j'' = j'{jtxns=map fixtransaction ts, jmarketprices=map fixmarketprice mps}
-      j' = j{jcommoditystyles = canonicalStyles $ dbg8 "journalAmounts" $ journalAmounts j}
       fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
       fixposting p@Posting{pamount=a} = p{pamount=fixmixedamount a}
       fixmarketprice mp@MarketPrice{mpamount=a} = mp{mpamount=fixamount a}
       fixmixedamount (Mixed as) = Mixed $ map fixamount as
       fixamount a@Amount{acommodity=c} = a{astyle=journalCommodityStyle j' c}
 
--- | Given a list of amounts in parse order, build a map from commodities
--- to canonical display styles for amounts in that commodity.
-canonicalStyles :: [Amount] -> M.Map Commodity AmountStyle
-canonicalStyles amts = M.fromList commstyles
+-- | Get this journal's standard display style for the given
+-- commodity.  That is the style defined by the last corresponding
+-- commodity format directive if any, otherwise the style inferred
+-- from the posting amounts (or in some cases, price amounts) in this
+-- commodity if any, otherwise the default style.
+journalCommodityStyle :: Journal -> CommoditySymbol -> AmountStyle
+journalCommodityStyle j c =
+  headDef amountstyle{asprecision=2} $
+  catMaybes [
+     M.lookup c (jcommodities j) >>= cformat
+    ,M.lookup c $ jinferredcommodities j
+    ]
+
+-- | Infer a display format for each commodity based on the amounts parsed.
+-- "hledger... will use the format of the first posting amount in the
+-- commodity, and the highest precision of all posting amounts in the commodity."
+journalInferCommodityStyles :: Journal -> Journal
+journalInferCommodityStyles j =
+  j{jinferredcommodities =
+        commodityStylesFromAmounts $
+        dbg8 "journalChooseCommmodityStyles using amounts" $ journalAmounts j}
+
+-- | Given a list of amounts in parse order, build a map from their commodity names
+-- to standard commodity display formats.
+commodityStylesFromAmounts :: [Amount] -> M.Map CommoditySymbol AmountStyle
+commodityStylesFromAmounts amts = M.fromList commstyles
   where
     samecomm = \a1 a2 -> acommodity a1 == acommodity a2
     commamts = [(acommodity $ head as, as) | as <- groupBy samecomm $ sortBy (comparing acommodity) amts]
     commstyles = [(c, canonicalStyleFrom $ map astyle as) | (c,as) <- commamts]
 
--- Given an ordered list of amount styles for a commodity, build a canonical style.
+-- | Given an ordered list of amount styles, choose a canonical style.
+-- That is: the style of the first, and the
+-- maximum precision of all.
 canonicalStyleFrom :: [AmountStyle] -> AmountStyle
 canonicalStyleFrom [] = amountstyle
 canonicalStyleFrom ss@(first:_) =
   first{asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
   where
-    -- precision is the maximum of all precisions seen
-    prec = maximum $ map asprecision ss
-    -- find the first decimal point and the first digit group style seen,
-    -- or use defaults.
-    mdec  = Just $ headDef '.' $ catMaybes $ map asdecimalpoint ss
     mgrps = maybe Nothing Just $ headMay $ catMaybes $ map asdigitgroups ss
-
--- | Get this journal's canonical amount style for the given commodity, or the null style.
-journalCommodityStyle :: Journal -> Commodity -> AmountStyle
-journalCommodityStyle j c = M.findWithDefault amountstyle c $ jcommoditystyles j
+    -- precision is maximum of all precisions
+    prec = maximum $ map asprecision ss
+    mdec  = Just $ headDef '.' $ catMaybes $ map asdecimalpoint ss
+    -- precision is that of first amount with a decimal point
+    -- (mdec, prec) =
+    --   case filter (isJust . asdecimalpoint) ss of
+    --   (s:_) -> (asdecimalpoint s, asprecision s)
+    --   []    -> (Just '.', 0)
 
 -- -- | Apply this journal's historical price records to unpriced amounts where possible.
 -- journalApplyMarketPrices :: Journal -> Journal
@@ -560,41 +661,36 @@ journalCommodityStyle j c = M.findWithDefault amountstyle c $ jcommoditystyles j
 
 -- -- | Get the price for a commodity on the specified day from the price database, if known.
 -- -- Does only one lookup step, ie will not look up the price of a price.
--- journalMarketPriceFor :: Journal -> Day -> Commodity -> Maybe MixedAmount
--- journalMarketPriceFor j d Commodity{symbol=s} = do
+-- journalMarketPriceFor :: Journal -> Day -> CommoditySymbol -> Maybe MixedAmount
+-- journalMarketPriceFor j d CommoditySymbol{symbol=s} = do
 --   let ps = reverse $ filter ((<= d).mpdate) $ filter ((s==).hsymbol) $ sortBy (comparing mpdate) $ jmarketprices j
 --   case ps of (MarketPrice{mpamount=a}:_) -> Just a
 --              _ -> Nothing
 
--- | Close any open timelog sessions in this journal using the provided current time.
-journalCloseTimeLogEntries :: LocalTime -> Journal -> Journal
-journalCloseTimeLogEntries now j@Journal{jtxns=ts, open_timelog_entries=es} =
-  j{jtxns = ts ++ (timeLogEntriesToTransactions now es), open_timelog_entries = []}
-
 -- | Convert all this journal's amounts to cost by applying their prices, if any.
 journalConvertAmountsToCost :: Journal -> Journal
 journalConvertAmountsToCost j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
     where
-      -- similar to journalCanonicaliseAmounts
+      -- similar to journalApplyCommodityStyles
       fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
       fixposting p@Posting{pamount=a} = p{pamount=fixmixedamount a}
       fixmixedamount (Mixed as) = Mixed $ map fixamount as
-      fixamount = canonicaliseAmount (jcommoditystyles j) . costOfAmount
+      fixamount = canonicaliseAmount (jinferredcommodities j) . costOfAmount
 
 -- -- | Get this journal's unique, display-preference-canonicalised commodities, by symbol.
--- journalCanonicalCommodities :: Journal -> M.Map String Commodity
+-- journalCanonicalCommodities :: Journal -> M.Map String CommoditySymbol
 -- journalCanonicalCommodities j = canonicaliseCommodities $ journalAmountCommodities j
 
 -- -- | Get all this journal's amounts' commodities, in the order parsed.
--- journalAmountCommodities :: Journal -> [Commodity]
+-- journalAmountCommodities :: Journal -> [CommoditySymbol]
 -- journalAmountCommodities = map acommodity . concatMap amounts . journalAmounts
 
 -- -- | Get all this journal's amount and price commodities, in the order parsed.
--- journalAmountAndPriceCommodities :: Journal -> [Commodity]
+-- journalAmountAndPriceCommodities :: Journal -> [CommoditySymbol]
 -- journalAmountAndPriceCommodities = concatMap amountCommodities . concatMap amounts . journalAmounts
 
 -- -- | Get this amount's commodity and any commodities referenced in its price.
--- amountCommodities :: Amount -> [Commodity]
+-- amountCommodities :: Amount -> [CommoditySymbol]
 -- amountCommodities Amount{acommodity=c,aprice=p} =
 --     case p of Nothing -> [c]
 --               Just (UnitPrice ma)  -> c:(concatMap amountCommodities $ amounts ma)
