@@ -7,12 +7,14 @@ tags.
 
 -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Hledger.Data.Transaction (
   -- * Transaction
   nullsourcepos,
   nulltransaction,
   txnTieKnot,
-  -- settxn,
+  txnUntieKnot,
   -- * operations
   showAccountName,
   hasRealPostings,
@@ -31,12 +33,15 @@ module Hledger.Data.Transaction (
   showTransaction,
   showTransactionUnelided,
   showTransactionUnelidedOneLineAmounts,
+  showPostingLine,
   -- * misc.
   tests_Hledger_Data_Transaction
 )
 where
 import Data.List
 import Data.Maybe
+import Data.Text (Text)
+import qualified Data.Text as T
 import Data.Time.Calendar
 import Test.HUnit
 import Text.Printf
@@ -51,10 +56,10 @@ import Hledger.Data.Amount
 instance Show Transaction where show = showTransactionUnelided
 
 instance Show ModifierTransaction where
-    show t = "= " ++ mtvalueexpr t ++ "\n" ++ unlines (map show (mtpostings t))
+    show t = "= " ++ T.unpack (mtvalueexpr t) ++ "\n" ++ unlines (map show (mtpostings t))
 
 instance Show PeriodicTransaction where
-    show t = "~ " ++ ptperiodicexpr t ++ "\n" ++ unlines (map show (ptpostings t))
+    show t = "~ " ++ T.unpack (ptperiodicexpr t) ++ "\n" ++ unlines (map show (ptpostings t))
 
 nullsourcepos :: GenericSourcePos
 nullsourcepos = GenericSourcePos "" 1 1
@@ -145,16 +150,16 @@ showTransactionHelper elide onelineamounts t =
       status | tstatus t == Cleared = " *"
              | tstatus t == Pending = " !"
              | otherwise            = ""
-      code = if length (tcode t) > 0 then printf " (%s)" $ tcode t else ""
-      desc = if null d then "" else " " ++ d where d = tdescription t
+      code = if T.length (tcode t) > 0 then printf " (%s)" $ T.unpack $ tcode t else ""
+      desc = if null d then "" else " " ++ d where d = T.unpack $ tdescription t
       (samelinecomment, newlinecomments) =
         case renderCommentLines (tcomment t) of []   -> ("",[])
                                                 c:cs -> (c,cs)
 
 -- Render a transaction or posting's comment as indented, semicolon-prefixed comment lines.
-renderCommentLines :: String -> [String]
-renderCommentLines s  = case lines s of ("":ls) -> "":map commentprefix ls
-                                        ls      -> map commentprefix ls
+renderCommentLines :: Text -> [String]
+renderCommentLines t  = case lines $ T.unpack t of ("":ls) -> "":map commentprefix ls
+                                                   ls      -> map commentprefix ls
     where
       commentprefix = indent . ("; "++)
 
@@ -187,7 +192,7 @@ postingAsLines elideamount onelineamounts ps p =
         showstatus p ++ fitString (Just acctwidth) Nothing False True (showAccountName Nothing (ptype p) (paccount p))
         where
           showstatus p = if pstatus p == Cleared then "* " else ""
-          acctwidth = maximum $ map (strWidth . paccount) ps
+          acctwidth = maximum $ map (textWidth . paccount) ps
 
     -- currently prices are considered part of the amount string when right-aligning amounts
     amount
@@ -201,6 +206,15 @@ postingAsLines elideamount onelineamounts ps p =
       case renderCommentLines (pcomment p) of []   -> ("",[])
                                               c:cs -> (c,cs)
 
+
+-- used in balance assertion error
+showPostingLine p =
+  indent $
+  if pstatus p == Cleared then "* " else "" ++
+  showAccountName Nothing (ptype p) (paccount p) ++
+  "    " ++
+  showMixedAmountOneLine (pamount p)
+
 tests_postingAsLines = [
    "postingAsLines" ~: do
     let p `gives` ls = assertEqual "" ls (postingAsLines False False [p] p)
@@ -229,12 +243,16 @@ indent = ("    "++)
 showAccountName :: Maybe Int -> PostingType -> AccountName -> String
 showAccountName w = fmt
     where
-      fmt RegularPosting = take w'
-      fmt VirtualPosting = parenthesise . reverse . take (w'-2) . reverse
-      fmt BalancedVirtualPosting = bracket . reverse . take (w'-2) . reverse
+      fmt RegularPosting = take w' . T.unpack
+      fmt VirtualPosting = parenthesise . reverse . take (w'-2) . reverse . T.unpack
+      fmt BalancedVirtualPosting = bracket . reverse . take (w'-2) . reverse . T.unpack
       w' = fromMaybe 999999 w
-      parenthesise s = "("++s++")"
-      bracket s = "["++s++"]"
+
+parenthesise :: String -> String
+parenthesise s = "("++s++")"
+
+bracket :: String -> String
+bracket s = "["++s++"]"
 
 hasRealPostings :: Transaction -> Bool
 hasRealPostings = not . null . realPostings
@@ -260,7 +278,7 @@ transactionPostingBalances t = (sumPostings $ realPostings t
 -- | Is this transaction balanced ? A balanced transaction's real
 -- (non-virtual) postings sum to 0, and any balanced virtual postings
 -- also sum to 0.
-isTransactionBalanced :: Maybe (Map.Map Commodity AmountStyle) -> Transaction -> Bool
+isTransactionBalanced :: Maybe (Map.Map CommoditySymbol AmountStyle) -> Transaction -> Bool
 isTransactionBalanced styles t =
     -- isReallyZeroMixedAmountCost rsum && isReallyZeroMixedAmountCost bvsum
     isZeroMixedAmount rsum' && isZeroMixedAmount bvsum'
@@ -274,7 +292,7 @@ isTransactionBalanced styles t =
 -- amount or conversion price(s), or return an error message.
 -- Balancing is affected by commodity display precisions, so those can
 -- (optionally) be provided.
-balanceTransaction :: Maybe (Map.Map Commodity AmountStyle) -> Transaction -> Either String Transaction
+balanceTransaction :: Maybe (Map.Map CommoditySymbol AmountStyle) -> Transaction -> Either String Transaction
 balanceTransaction styles t =
   case inferBalancingAmount t of
     Left err -> Left err
@@ -402,11 +420,16 @@ transactionDate2 t = fromMaybe (tdate t) $ tdate2 t
 -- | Ensure a transaction's postings refer back to it, so that eg
 -- relatedPostings works right.
 txnTieKnot :: Transaction -> Transaction
-txnTieKnot t@Transaction{tpostings=ps} = t{tpostings=map (settxn t) ps}
+txnTieKnot t@Transaction{tpostings=ps} = t{tpostings=map (postingSetTransaction t) ps}
+
+-- | Ensure a transaction's postings do not refer back to it, so that eg
+-- recursiveSize and GHCI's :sprint work right.
+txnUntieKnot :: Transaction -> Transaction
+txnUntieKnot t@Transaction{tpostings=ps} = t{tpostings=map (\p -> p{ptransaction=Nothing}) ps}
 
 -- | Set a posting's parent transaction.
-settxn :: Transaction -> Posting -> Posting
-settxn t p = p{ptransaction=Just t}
+postingSetTransaction :: Transaction -> Posting -> Posting
+postingSetTransaction t p = p{ptransaction=Just t}
 
 tests_Hledger_Data_Transaction = TestList $ concat [
   tests_postingAsLines,
