@@ -1,4 +1,3 @@
-{-# LANGUAGE ScopedTypeVariables #-}
 {-|
 
 This is the entry point to hledger's reading system, which can read
@@ -8,7 +7,12 @@ to import modules below this one.
 
 -}
 
-module Hledger.Read (
+{-# LANGUAGE ScopedTypeVariables, OverloadedStrings #-}
+
+module Hledger.Read
+  (
+       module Hledger.Read.Common,
+       readFormatNames,
        -- * Journal reading API
        defaultJournalPath,
        defaultJournal,
@@ -20,12 +24,12 @@ module Hledger.Read (
        ensureJournalFileExists,
        -- * Parsers used elsewhere
        postingp,
-       accountnamep,
-       amountp,
-       amountp',
-       mamountp',
-       numberp,
-       codep,
+       -- accountnamep,
+       -- amountp,
+       -- amountp',
+       -- mamountp',
+       -- numberp,
+       -- codep,
        accountaliasp,
        -- * Tests
        samplejournal,
@@ -36,40 +40,143 @@ import qualified Control.Exception as C
 import Control.Monad.Except
 import Data.List
 import Data.Maybe
+import Data.Text (Text)
+import qualified Data.Text as T
 import System.Directory (doesFileExist, getHomeDirectory)
 import System.Environment (getEnv)
 import System.Exit (exitFailure)
 import System.FilePath ((</>))
-import System.IO (IOMode(..), openFile, stdin, stderr, hSetNewlineMode, universalNewlineMode)
+import System.IO (stderr)
 import Test.HUnit
 import Text.Printf
 
 import Hledger.Data.Dates (getCurrentDay)
 import Hledger.Data.Types
-import Hledger.Data.Journal (nullctx)
+import Hledger.Read.Common
 import Hledger.Read.JournalReader as JournalReader
-import Hledger.Read.TimelogReader as TimelogReader
+import Hledger.Read.TimedotReader as TimedotReader
+import Hledger.Read.TimeclockReader as TimeclockReader
 import Hledger.Read.CsvReader as CsvReader
 import Hledger.Utils
 import Prelude hiding (getContents, writeFile)
-import Hledger.Utils.UTF8IOCompat (hGetContents, writeFile)
+import Hledger.Utils.UTF8IOCompat (writeFile)
 
 
-journalEnvVar           = "LEDGER_FILE"
-journalEnvVar2          = "LEDGER"
-journalDefaultFilename  = ".hledger.journal"
-
 -- The available data file readers, each one handling a particular data
 -- format. The first is also used as the default for unknown formats.
 readers :: [Reader]
 readers = [
   JournalReader.reader
- ,TimelogReader.reader
+ ,TimeclockReader.reader
+ ,TimedotReader.reader
  ,CsvReader.reader
  ]
 
--- | All the data formats we can read.
--- formats = map rFormat readers
+readFormatNames :: [StorageFormat]
+readFormatNames = map rFormat readers
+
+journalEnvVar           = "LEDGER_FILE"
+journalEnvVar2          = "LEDGER"
+journalDefaultFilename  = ".hledger.journal"
+
+-- | Which readers are worth trying for this (possibly unspecified) format, filepath, and data ?
+readersFor :: (Maybe StorageFormat, Maybe FilePath, Text) -> [Reader]
+readersFor (format,path,t) =
+    dbg1 ("possible readers for "++show (format,path,textElideRight 30 t)) $
+    case format of
+     Just f  -> case readerForStorageFormat f of Just r  -> [r]
+                                                 Nothing -> []
+     Nothing -> case path of Nothing  -> readers
+                             Just p   -> case readersForPathAndData (p,t) of [] -> readers
+                                                                             rs -> rs
+
+-- | Find the (first) reader which can handle the given format, if any.
+readerForStorageFormat :: StorageFormat -> Maybe Reader
+readerForStorageFormat s | null rs = Nothing
+                  | otherwise = Just $ head rs
+    where
+      rs = filter ((s==).rFormat) readers :: [Reader]
+
+-- | Find the readers which think they can handle the given file path and data, if any.
+readersForPathAndData :: (FilePath,Text) -> [Reader]
+readersForPathAndData (f,t) = filter (\r -> dbg1 ("try "++rFormat r++" format") $ (rDetector r) f t) readers
+
+-- try each reader in turn, returning the error of the first if all fail
+tryReaders :: [Reader] -> Maybe FilePath -> Bool -> Maybe FilePath -> Text -> IO (Either String Journal)
+tryReaders readers mrulesfile assrt path t = firstSuccessOrBestError [] readers
+  where
+    firstSuccessOrBestError :: [String] -> [Reader] -> IO (Either String Journal)
+    firstSuccessOrBestError [] []        = return $ Left "no readers found"
+    firstSuccessOrBestError errs (r:rs) = do
+      dbg1IO "trying reader" (rFormat r)
+      result <- (runExceptT . (rParser r) mrulesfile assrt path') t
+      dbg1IO "reader result" $ either id show result
+      case result of Right j -> return $ Right j                       -- success!
+                     Left e  -> firstSuccessOrBestError (errs++[e]) rs -- keep trying
+    firstSuccessOrBestError (e:_) []    = return $ Left e              -- none left, return first error
+    path' = fromMaybe "(string)" path
+
+-- | Read a journal from this string, trying whatever readers seem appropriate:
+--
+-- - if a format is specified, try that reader only
+--
+-- - or if one or more readers recognises the file path and data, try those
+--
+-- - otherwise, try them all.
+--
+-- A CSV conversion rules file may also be specified for use by the CSV reader.
+-- Also there is a flag specifying whether to check or ignore balance assertions in the journal.
+readJournal :: Maybe StorageFormat -> Maybe FilePath -> Bool -> Maybe FilePath -> Text -> IO (Either String Journal)
+readJournal mformat mrulesfile assrt mpath t = tryReaders (readersFor (mformat, mpath, t)) mrulesfile assrt mpath t
+
+-- | Call readJournalFile on each specified file path, and combine the
+-- resulting journals into one. If there are any errors, the first is
+-- returned, otherwise they are combined per Journal's monoid instance
+-- (concatenated, basically). Parse context (eg directives & aliases)
+-- is not maintained across file boundaries, it resets at the start of
+-- each file (though the rfinal parse state saved in the resulting
+-- journal is the combination of parse states from all files).
+readJournalFiles :: Maybe StorageFormat -> Maybe FilePath -> Bool -> [FilePath] -> IO (Either String Journal)
+readJournalFiles mformat mrulesfile assrt fs = do
+  (either Left (Right . mconcat) . sequence)
+    <$> mapM (readJournalFile mformat mrulesfile assrt) fs
+
+-- | Read a Journal from this file (or stdin if the filename is -) or give
+-- an error message, using the specified data format or trying all known
+-- formats. A CSV conversion rules file may be specified for better
+-- conversion of that format. Also there is a flag specifying whether
+-- to check or ignore balance assertions in the journal.
+readJournalFile :: Maybe StorageFormat -> Maybe FilePath -> Bool -> FilePath -> IO (Either String Journal)
+readJournalFile mformat mrulesfile assrt f = do
+  requireJournalFileExists f
+  readFileOrStdinAnyLineEnding f >>= readJournal mformat mrulesfile assrt (Just f)
+
+-- | If the specified journal file does not exist, give a helpful error and quit.
+requireJournalFileExists :: FilePath -> IO ()
+requireJournalFileExists "-" = return ()
+requireJournalFileExists f = do
+  exists <- doesFileExist f
+  when (not exists) $ do  -- XXX might not be a journal file
+    hPrintf stderr "The hledger journal file \"%s\" was not found.\n" f
+    hPrintf stderr "Please create it first, eg with \"hledger add\" or a text editor.\n"
+    hPrintf stderr "Or, specify an existing journal file with -f or LEDGER_FILE.\n"
+    exitFailure
+
+-- | Ensure there is a journal file at the given path, creating an empty one if needed.
+ensureJournalFileExists :: FilePath -> IO ()
+ensureJournalFileExists f = do
+  exists <- doesFileExist f
+  when (not exists) $ do
+    hPrintf stderr "Creating hledger journal file %s.\n" f
+    -- note Hledger.Utils.UTF8.* do no line ending conversion on windows,
+    -- we currently require unix line endings on all platforms.
+    newJournalContent >>= writeFile f
+
+-- | Give the content for a new auto-created journal file.
+newJournalContent :: IO String
+newJournalContent = do
+  d <- getCurrentDay
+  return $ printf "; journal created %s by hledger\n" (show d)
 
 -- | Get the default journal file path specified by the environment.
 -- Like ledger, we look first for the LEDGER_FILE environment
@@ -95,9 +202,9 @@ defaultJournalPath = do
 defaultJournal :: IO Journal
 defaultJournal = defaultJournalPath >>= readJournalFile Nothing Nothing True >>= either error' return
 
--- | Read a journal from the given string, trying all known formats, or simply throw an error.
-readJournal' :: String -> IO Journal
-readJournal' s = readJournal Nothing Nothing True Nothing s >>= either error' return
+-- | Read a journal from the given text, trying all known formats, or simply throw an error.
+readJournal' :: Text -> IO Journal
+readJournal' t = readJournal Nothing Nothing True Nothing t >>= either error' return
 
 tests_readJournal' = [
   "readJournal' parses sample journal" ~: do
@@ -105,110 +212,9 @@ tests_readJournal' = [
      assertBool "" True
  ]
 
-
-
--- | Read a journal from this string, trying whatever readers seem appropriate:
---
--- - if a format is specified, try that reader only
---
--- - or if one or more readers recognises the file path and data, try those
---
--- - otherwise, try them all.
---
--- A CSV conversion rules file may also be specified for use by the CSV reader.
--- Also there is a flag specifying whether to check or ignore balance assertions in the journal.
-readJournal :: Maybe StorageFormat -> Maybe FilePath -> Bool -> Maybe FilePath -> String -> IO (Either String Journal)
-readJournal format rulesfile assrt path s =
-  tryReaders $ readersFor (format, path, s)
-  where
-    -- try each reader in turn, returning the error of the first if all fail
-    tryReaders :: [Reader] -> IO (Either String Journal)
-    tryReaders = firstSuccessOrBestError []
-      where
-        firstSuccessOrBestError :: [String] -> [Reader] -> IO (Either String Journal)
-        firstSuccessOrBestError [] []        = return $ Left "no readers found"
-        firstSuccessOrBestError errs (r:rs) = do
-          dbg1IO "trying reader" (rFormat r)
-          result <- (runExceptT . (rParser r) rulesfile assrt path') s
-          dbg1IO "reader result" $ either id show result
-          case result of Right j -> return $ Right j                       -- success!
-                         Left e  -> firstSuccessOrBestError (errs++[e]) rs -- keep trying
-        firstSuccessOrBestError (e:_) []    = return $ Left e              -- none left, return first error
-        path' = fromMaybe "(string)" path
-
--- | Which readers are worth trying for this (possibly unspecified) format, filepath, and data ?
-readersFor :: (Maybe StorageFormat, Maybe FilePath, String) -> [Reader]
-readersFor (format,path,s) =
-    dbg1 ("possible readers for "++show (format,path,elideRight 30 s)) $
-    case format of
-     Just f  -> case readerForStorageFormat f of Just r  -> [r]
-                                                 Nothing -> []
-     Nothing -> case path of Nothing  -> readers
-                             Just p   -> case readersForPathAndData (p,s) of [] -> readers
-                                                                             rs -> rs
-
--- | Find the (first) reader which can handle the given format, if any.
-readerForStorageFormat :: StorageFormat -> Maybe Reader
-readerForStorageFormat s | null rs = Nothing
-                  | otherwise = Just $ head rs
-    where
-      rs = filter ((s==).rFormat) readers :: [Reader]
-
--- | Find the readers which think they can handle the given file path and data, if any.
-readersForPathAndData :: (FilePath,String) -> [Reader]
-readersForPathAndData (f,s) = filter (\r -> (rDetector r) f s) readers
-
--- | Read a Journal from this file (or stdin if the filename is -) or give
--- an error message, using the specified data format or trying all known
--- formats. A CSV conversion rules file may be specified for better
--- conversion of that format. Also there is a flag specifying whether
--- to check or ignore balance assertions in the journal.
-readJournalFile :: Maybe StorageFormat -> Maybe FilePath -> Bool -> FilePath -> IO (Either String Journal)
-readJournalFile format rulesfile assrt f = readJournalFiles format rulesfile assrt [f]
-
-readJournalFiles :: Maybe StorageFormat -> Maybe FilePath -> Bool -> [FilePath] -> IO (Either String Journal)
-readJournalFiles format rulesfile assrt f = do
-  contents <- fmap concat $ mapM readFileAnyNewline f
-  readJournal format rulesfile assrt (listToMaybe f) contents
- where
-  readFileAnyNewline f = do
-    requireJournalFileExists f
-    h <- fileHandle f
-    hSetNewlineMode h universalNewlineMode
-    hGetContents h
-  fileHandle "-" = return stdin
-  fileHandle f = openFile f ReadMode
-
--- | If the specified journal file does not exist, give a helpful error and quit.
-requireJournalFileExists :: FilePath -> IO ()
-requireJournalFileExists "-" = return ()
-requireJournalFileExists f = do
-  exists <- doesFileExist f
-  when (not exists) $ do
-    hPrintf stderr "The hledger journal file \"%s\" was not found.\n" f
-    hPrintf stderr "Please create it first, eg with \"hledger add\" or a text editor.\n"
-    hPrintf stderr "Or, specify an existing journal file with -f or LEDGER_FILE.\n"
-    exitFailure
-
--- | Ensure there is a journal file at the given path, creating an empty one if needed.
-ensureJournalFileExists :: FilePath -> IO ()
-ensureJournalFileExists f = do
-  exists <- doesFileExist f
-  when (not exists) $ do
-    hPrintf stderr "Creating hledger journal file %s.\n" f
-    -- note Hledger.Utils.UTF8.* do no line ending conversion on windows,
-    -- we currently require unix line endings on all platforms.
-    newJournalContent >>= writeFile f
-
--- | Give the content for a new auto-created journal file.
-newJournalContent :: IO String
-newJournalContent = do
-  d <- getCurrentDay
-  return $ printf "; journal created %s by hledger\n" (show d)
-
 -- tests
 
-samplejournal = readJournal' $ unlines
+samplejournal = readJournal' $ T.unlines
  ["2008/01/01 income"
  ,"    assets:bank:checking  $1"
  ,"    income:salary"
@@ -240,11 +246,12 @@ tests_Hledger_Read = TestList $
   tests_readJournal'
   ++ [
    tests_Hledger_Read_JournalReader,
-   tests_Hledger_Read_TimelogReader,
+   tests_Hledger_Read_TimeclockReader,
+   tests_Hledger_Read_TimedotReader,
    tests_Hledger_Read_CsvReader,
 
    "journal" ~: do
-    r <- runExceptT $ parseWithCtx nullctx JournalReader.journalp ""
+    r <- runExceptT $ parseWithState mempty JournalReader.journalp ""
     assertBool "journalp should parse an empty file" (isRight $ r)
     jE <- readJournal Nothing Nothing True Nothing "" -- don't know how to get it from journal
     either error' (assertBool "journalp parsing an empty file should give an empty journal" . null . jtxns) jE
