@@ -1,4 +1,4 @@
-{-# LANGUAGE RecordWildCards, DeriveDataTypeable, FlexibleInstances, ScopedTypeVariables #-}
+{-# LANGUAGE FlexibleInstances, ScopedTypeVariables #-}
 {-|
 
 Multi-column balance reports, used by the balance command.
@@ -9,7 +9,8 @@ module Hledger.Reports.MultiBalanceReports (
   MultiBalanceReport(..),
   MultiBalanceReportRow,
   multiBalanceReport,
-  multiBalanceReportValue
+  multiBalanceReportValue,
+  singleBalanceReport
 
   -- -- * Tests
   -- tests_Hledger_Reports_MultiBalanceReport
@@ -32,32 +33,34 @@ import Hledger.Reports.BalanceReport
 
 -- | A multi balance report is a balance report with one or more columns. It has:
 --
--- 1. a list of each column's date span
+-- 1. a list of each column's period (date span)
 --
--- 2. a list of rows, each containing a renderable account name and the amounts to show in each column
+-- 2. a list of row items, each containing:
 --
--- 3. a list of each column's final total
+--   * the full account name
+--
+--   * the leaf account name
+--
+--   * the account's depth
+--
+--   * the amounts to show in each column
+--
+--   * the total of the row's amounts
+--
+--   * the average of the row's amounts
+--
+-- 3. the column totals and the overall total and average
 --
 -- The meaning of the amounts depends on the type of multi balance
 -- report, of which there are three: periodic, cumulative and historical
 -- (see 'BalanceType' and "Hledger.Cli.Balance").
-newtype MultiBalanceReport = MultiBalanceReport ([DateSpan]
-                                                ,[MultiBalanceReportRow]
-                                                ,MultiBalanceTotalsRow
-                                                )
-
--- | A row in a multi balance report has
---
--- * An account name, with rendering hints
---
--- * A list of amounts to be shown in each of the report's columns.
---
--- * The total of the row amounts.
---
--- * The average of the row amounts.
-type MultiBalanceReportRow = (RenderableAccountName, [MixedAmount], MixedAmount, MixedAmount)
-
-type MultiBalanceTotalsRow = ([MixedAmount], MixedAmount, MixedAmount)
+newtype MultiBalanceReport =
+  MultiBalanceReport ([DateSpan]
+                     ,[MultiBalanceReportRow]
+                     ,MultiBalanceReportTotals
+                     )
+type MultiBalanceReportRow    = (AccountName, AccountName, Int, [MixedAmount], MixedAmount, MixedAmount)
+type MultiBalanceReportTotals = ([MixedAmount], MixedAmount, MixedAmount)
 
 instance Show MultiBalanceReport where
     -- use ppShow to break long lists onto multiple lines
@@ -69,6 +72,20 @@ instance Show MultiBalanceReport where
 -- type alias just to remind us which AccountNames might be depth-clipped, below.
 type ClippedAccountName = AccountName
 
+-- | Generates a single column BalanceReport like balanceReport, but uses
+-- multiBalanceReport, so supports --historical. Does not support
+-- boring parent eliding yet.
+singleBalanceReport :: ReportOpts -> Query -> Journal -> BalanceReport
+singleBalanceReport opts q j = (rows', total)
+  where
+    MultiBalanceReport (_, rows, (totals, _, _)) = multiBalanceReport opts q j
+    rows' = [(a
+             ,if tree_ opts then a' else a   -- BalanceReport expects full account name here with --flat
+             ,if tree_ opts then d-1 else 0  -- BalanceReport uses 0-based account depths
+             , headDef nullmixedamt amts     -- 0 columns is illegal, should not happen, return zeroes if it does
+             ) | (a,a',d, amts, _, _) <- rows]
+    total = headDef nullmixedamt totals
+
 -- | Generate a multicolumn balance report for the matched accounts,
 -- showing the change of balance, accumulated balance, or historical balance
 -- in each of the specified periods.
@@ -84,7 +101,7 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
       precedingq = dbg1 "precedingq" $ And [datelessq, dateqcons $ DateSpan Nothing (spanStart reportspan)]
       requestedspan  = dbg1 "requestedspan"  $ queryDateSpan (date2_ opts) q                              -- span specified by -b/-e/-p options and query args
       requestedspan' = dbg1 "requestedspan'" $ requestedspan `spanDefaultsFrom` journalDateSpan (date2_ opts) j  -- if open-ended, close it using the journal's end dates
-      intervalspans  = dbg1 "intervalspans"  $ splitSpan (intervalFromOpts opts) requestedspan'           -- interval spans enclosing it
+      intervalspans  = dbg1 "intervalspans"  $ splitSpan (interval_ opts) requestedspan'           -- interval spans enclosing it
       reportspan     = dbg1 "reportspan"     $ DateSpan (maybe Nothing spanStart $ headMay intervalspans) -- the requested span enlarged to a whole number of intervals
                                                        (maybe Nothing spanEnd   $ lastMay intervalspans)
       newdatesq = dbg1 "newdateq" $ dateqcons reportspan
@@ -97,15 +114,15 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
           filterJournalPostings reportq $        -- remove postings not matched by (adjusted) query
           journalSelectingAmountFromOpts opts j
 
-      displayspans = dbg1 "displayspans" $ splitSpan (intervalFromOpts opts) displayspan
+      displayspans = dbg1 "displayspans" $ splitSpan (interval_ opts) displayspan
         where
           displayspan
-            | empty_ opts = dbg1 "displayspan (-E)" $ reportspan                                -- all the requested intervals
+            | empty_ opts = dbg1 "displayspan (-E)" reportspan                                -- all the requested intervals
             | otherwise   = dbg1 "displayspan"      $ requestedspan `spanIntersect` matchedspan -- exclude leading/trailing empty intervals
           matchedspan = dbg1 "matchedspan" $ postingsDateSpan' (whichDateFromOpts opts) ps
 
       psPerSpan :: [[Posting]] =
-          dbg1 "psPerSpan" $
+          dbg1 "psPerSpan"
           [filter (isPostingInDateSpan' (whichDateFromOpts opts) s) ps | s <- displayspans]
 
       postedAcctBalChangesPerSpan :: [[(ClippedAccountName, MixedAmount)]] =
@@ -125,7 +142,7 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
       postedAccts :: [AccountName] = dbg1 "postedAccts" $ sort $ accountNamesFromPostings ps
 
       -- starting balances and accounts from transactions before the report start date
-      startacctbals = dbg1 "startacctbals" $ map (\((a,_,_),b) -> (a,b)) startbalanceitems
+      startacctbals = dbg1 "startacctbals" $ map (\(a,_,_,b) -> (a,b)) startbalanceitems
           where
             (startbalanceitems,_) = dbg1 "starting balance report" $ balanceReport opts' precedingq j
                                     where
@@ -141,22 +158,22 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
           if empty_ opts then nub $ sort $ startAccts ++ postedAccts else postedAccts
 
       acctBalChangesPerSpan :: [[(ClippedAccountName, MixedAmount)]] =
-          dbg1 "acctBalChangesPerSpan" $
+          dbg1 "acctBalChangesPerSpan"
           [sortBy (comparing fst) $ unionBy (\(a,_) (a',_) -> a == a') postedacctbals zeroes
            | postedacctbals <- postedAcctBalChangesPerSpan]
           where zeroes = [(a, nullmixedamt) | a <- displayedAccts]
 
       acctBalChanges :: [(ClippedAccountName, [MixedAmount])] =
-          dbg1 "acctBalChanges" $
+          dbg1 "acctBalChanges"
           [(a, map snd abs) | abs@((a,_):_) <- transpose acctBalChangesPerSpan] -- never null, or used when null...
 
       items :: [MultiBalanceReportRow] =
-          dbg1 "items" $
-          [((a, accountLeafName a, accountNameLevel a), displayedBals, rowtot, rowavg)
+          dbg1 "items"
+          [(a, accountLeafName a, accountNameLevel a, displayedBals, rowtot, rowavg)
            | (a,changes) <- acctBalChanges
            , let displayedBals = case balancetype_ opts of
                                   HistoricalBalance -> drop 1 $ scanl (+) (startingBalanceFor a) changes
-                                  CumulativeBalance -> drop 1 $ scanl (+) nullmixedamt changes
+                                  CumulativeChange -> drop 1 $ scanl (+) nullmixedamt changes
                                   _                 -> changes
            , let rowtot = sum displayedBals
            , let rowavg = averageMixedAmounts displayedBals
@@ -167,13 +184,13 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
           -- dbg1 "totals" $
           map sum balsbycol
           where
-            balsbycol = transpose [bs | ((a,_,_),bs,_,_) <- items, not (tree_ opts) || a `elem` highestlevelaccts]
+            balsbycol = transpose [bs | (a,_,_,bs,_,_) <- items, not (tree_ opts) || a `elem` highestlevelaccts]
             highestlevelaccts     =
-                dbg1 "highestlevelaccts" $
+                dbg1 "highestlevelaccts"
                 [a | a <- displayedAccts, not $ any (`elem` displayedAccts) $ init $ expandAccountName a]
 
-      totalsrow :: MultiBalanceTotalsRow =
-          dbg1 "totalsrow" $
+      totalsrow :: MultiBalanceReportTotals =
+          dbg1 "totalsrow"
           (totals, sum totals, averageMixedAmounts totals)
 
       dbg1 s = let p = "multiBalanceReport" in Hledger.Utils.dbg1 (p++" "++s)  -- add prefix in this function's debug output
@@ -188,7 +205,7 @@ multiBalanceReportValue j d r = r'
     MultiBalanceReport (spans, rows, (coltotals, rowtotaltotal, rowavgtotal)) = r
     r' = MultiBalanceReport
          (spans,
-          [(n, map convert rowamts, convert rowtotal, convert rowavg) | (n, rowamts, rowtotal, rowavg) <- rows],
+          [(acct, acct', depth, map convert rowamts, convert rowtotal, convert rowavg) | (acct, acct', depth, rowamts, rowtotal, rowavg) <- rows],
           (map convert coltotals, convert rowtotaltotal, convert rowavgtotal))
     convert = mixedAmountValue j d
 
