@@ -22,6 +22,7 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           module Hledger.Utils.Parse,
                           module Hledger.Utils.Regex,
                           module Hledger.Utils.String,
+                          module Hledger.Utils.Text,
                           module Hledger.Utils.Test,
                           module Hledger.Utils.Tree,
                           -- Debug.Trace.trace,
@@ -32,13 +33,16 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           )
 where
 import Control.Monad (liftM)
-import Control.Monad.IO.Class (MonadIO, liftIO)
 -- import Data.Char
 -- import Data.List
 -- import Data.Maybe
 -- import Data.PPrint
+import Data.Text (Text)
+import qualified Data.Text.IO as T
 import Data.Time.Clock
 import Data.Time.LocalTime
+-- import Data.Text (Text)
+-- import qualified Data.Text as T
 import System.Directory (getHomeDirectory)
 import System.FilePath((</>), isRelative)
 import System.IO
@@ -49,6 +53,7 @@ import Hledger.Utils.Debug
 import Hledger.Utils.Parse
 import Hledger.Utils.Regex
 import Hledger.Utils.String
+import Hledger.Utils.Text
 import Hledger.Utils.Test
 import Hledger.Utils.Tree
 -- import Prelude hiding (readFile,writeFile,appendFile,getContents,putStr,putStrLn)
@@ -92,6 +97,8 @@ splitAtElement x l =
     split es = let (first,rest) = break (x==) es
                in first : splitAtElement x rest
 
+-- text
+
 -- time
 
 getCurrentLocalTime :: IO LocalTime
@@ -100,6 +107,12 @@ getCurrentLocalTime = do
   tz <- getCurrentTimeZone
   return $ utcToLocalTime tz t
 
+getCurrentZonedTime :: IO ZonedTime
+getCurrentZonedTime = do
+  t <- getCurrentTime
+  tz <- getCurrentTimeZone
+  return $ utcToZonedTime tz t
+
 -- misc
 
 isLeft :: Either a b -> Bool
@@ -115,25 +128,44 @@ applyN n f = (!! n) . iterate f
 
 -- | Convert a possibly relative, possibly tilde-containing file path to an absolute one,
 -- given the current directory. ~username is not supported. Leave "-" unchanged.
-expandPath :: MonadIO m => FilePath -> FilePath -> m FilePath -- general type sig for use in reader parsers
+-- Can raise an error.
+expandPath :: FilePath -> FilePath -> IO FilePath -- general type sig for use in reader parsers
 expandPath _ "-" = return "-"
 expandPath curdir p = (if isRelative p then (curdir </>) else id) `liftM` expandPath' p
   where
-    expandPath' ('~':'/':p)  = liftIO $ (</> p) `fmap` getHomeDirectory
-    expandPath' ('~':'\\':p) = liftIO $ (</> p) `fmap` getHomeDirectory
-    expandPath' ('~':_)      = error' "~USERNAME in paths is not supported"
+    expandPath' ('~':'/':p)  = (</> p) <$> getHomeDirectory
+    expandPath' ('~':'\\':p) = (</> p) <$> getHomeDirectory
+    expandPath' ('~':_)      = ioError $ userError "~USERNAME in paths is not supported"
     expandPath' p            = return p
 
 firstJust ms = case dropWhile (==Nothing) ms of
     [] -> Nothing
     (md:_) -> md
 
--- | Read a file in universal newline mode, handling whatever newline convention it may contain.
-readFile' :: FilePath -> IO String
+-- | Read a file in universal newline mode, handling any of the usual line ending conventions.
+readFile' :: FilePath -> IO Text
 readFile' name =  do
   h <- openFile name ReadMode
   hSetNewlineMode h universalNewlineMode
-  hGetContents h
+  T.hGetContents h
+
+-- | Read a file in universal newline mode, handling any of the usual line ending conventions.
+readFileAnyLineEnding :: FilePath -> IO Text
+readFileAnyLineEnding path =  do
+  h <- openFile path ReadMode
+  hSetNewlineMode h universalNewlineMode
+  T.hGetContents h
+
+-- | Read the given file, or standard input if the path is "-", using
+-- universal newline mode.
+readFileOrStdinAnyLineEnding :: String -> IO Text
+readFileOrStdinAnyLineEnding f = do
+  h <- fileHandle f
+  hSetNewlineMode h universalNewlineMode
+  T.hGetContents h
+  where
+    fileHandle "-" = return stdin
+    fileHandle f = openFile f ReadMode
 
 -- | Total version of maximum, for integral types, giving 0 for an empty list.
 maximum' :: Integral a => [a] -> a
