@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, FlexibleContexts #-}
+{-# LANGUAGE CPP, FlexibleContexts, TypeFamilies #-}
 -- | Debugging helpers
 
 -- more:
@@ -16,50 +16,42 @@ module Hledger.Utils.Debug (
 )
 where
 
-import Control.Monad (when)
-import Data.List
-import Debug.Trace
-import Safe (readDef)
-import System.Environment (getArgs)
-import System.Exit
-import System.IO.Unsafe (unsafePerformIO)
-import Text.Parsec
-import Text.Printf
+import           Control.Monad (when)
+import           Control.Monad.IO.Class
+import           Data.List hiding (uncons)
+import qualified Data.Text as T
+import           Debug.Trace
+import           Hledger.Utils.Parse
+import           Safe (readDef)
+import           System.Environment (getArgs)
+import           System.Exit
+import           System.IO.Unsafe (unsafePerformIO)
+import           Text.Megaparsec
+import           Text.Printf
 
 #if __GLASGOW_HASKELL__ >= 704
-import Text.Show.Pretty (ppShow)
+import           Text.Show.Pretty (ppShow)
 #else
 -- the required pretty-show version requires GHC >= 7.4
 ppShow :: Show a => a -> String
 ppShow = show
 #endif
 
+pprint :: Show a => a -> IO ()
+pprint = putStrLn . ppShow
 
--- | Trace (print on stdout at runtime) a showable value.
--- (for easily tracing in the middle of a complex expression)
-strace :: Show a => a -> a
-strace a = trace (show a) a
-
--- | Labelled trace - like strace, with a label prepended.
-ltrace :: Show a => String -> a -> a
-ltrace l a = trace (l ++ ": " ++ show a) a
-
--- | Monadic trace - like strace, but works as a standalone line in a monad.
-mtrace :: (Monad m, Show a) => a -> m a
-mtrace a = strace a `seq` return a
-
--- | Custom trace - like strace, with a custom show function.
+-- | Trace (print to stderr) a showable value using a custom show function.
 traceWith :: (a -> String) -> a -> a
-traceWith f e = trace (f e) e
+traceWith f a = trace (f a) a
 
 -- | Parsec trace - show the current parsec position and next input,
 -- and the provided label if it's non-null.
-ptrace :: Stream [Char] m t => String -> ParsecT [Char] st m ()
+ptrace :: String -> TextParser m ()
 ptrace msg = do
   pos <- getPosition
-  next <- take peeklength `fmap` getInput
+  next <- (T.take peeklength) `fmap` getInput
   let (l,c) = (sourceLine pos, sourceColumn pos)
-      s  = printf "at line %2d col %2d: %s" l c (show next) :: String
+      s  = printf "at line %2d col %2d: %s" (unPos l) (unPos c) (show next) :: String
       s' = printf ("%-"++show (peeklength+30)++"s") s ++ " " ++ msg
   trace s' $ return ()
   where
@@ -70,8 +62,10 @@ ptrace msg = do
 -- @--debug@ command line flag sets it to 1, or @--debug=N@ sets it to
 -- a higher value (note: not @--debug N@ for some reason).  This uses
 -- unsafePerformIO and can be accessed from anywhere and before normal
--- command-line processing. After command-line processing, it is also
--- available as the @debug_@ field of 'Hledger.Cli.CliOptions.CliOpts'.
+-- command-line processing. When running with :main in GHCI, you must
+-- touch and reload this module to see the effect of a new --debug option.
+-- After command-line processing, it is also available as the @debug_@
+-- field of 'Hledger.Cli.CliOptions.CliOpts'.
 -- {-# OPTIONS_GHC -fno-cse #-} 
 -- {-# NOINLINE debugLevel #-}
 debugLevel :: Int
@@ -87,9 +81,11 @@ debugLevel = case snd $ break (=="--debug") args of
       args = unsafePerformIO getArgs
 
 -- | Convenience aliases for tracePrettyAt.
--- Pretty-print a message and the showable value to the console, then return it.
-dbg :: Show a => String -> a -> a
-dbg = tracePrettyAt 0
+
+-- Always pretty-print a message and the showable value to the console, then return it.
+-- ("dbg" without the 0 clashes with megaparsec 5.1).
+dbg0 :: Show a => String -> a -> a
+dbg0 = tracePrettyAt 0
 
 -- | Pretty-print a message and the showable value to the console when the debug level is >= 1, then return it. Uses unsafePerformIO.
 dbg1 :: Show a => String -> a -> a
@@ -121,34 +117,34 @@ dbg9 = tracePrettyAt 9
 
 -- | Convenience aliases for tracePrettyAtIO.
 -- Like dbg, but convenient to insert in an IO monad.
-dbgIO :: Show a => String -> a -> IO ()
+dbgIO :: (MonadIO m, Show a) => String -> a -> m ()
 dbgIO = tracePrettyAtIO 0
 
-dbg1IO :: Show a => String -> a -> IO ()
+dbg1IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg1IO = tracePrettyAtIO 1
 
-dbg2IO :: Show a => String -> a -> IO ()
+dbg2IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg2IO = tracePrettyAtIO 2
 
-dbg3IO :: Show a => String -> a -> IO ()
+dbg3IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg3IO = tracePrettyAtIO 3
 
-dbg4IO :: Show a => String -> a -> IO ()
+dbg4IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg4IO = tracePrettyAtIO 4
 
-dbg5IO :: Show a => String -> a -> IO ()
+dbg5IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg5IO = tracePrettyAtIO 5
 
-dbg6IO :: Show a => String -> a -> IO ()
+dbg6IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg6IO = tracePrettyAtIO 6
 
-dbg7IO :: Show a => String -> a -> IO ()
+dbg7IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg7IO = tracePrettyAtIO 7
 
-dbg8IO :: Show a => String -> a -> IO ()
+dbg8IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg8IO = tracePrettyAtIO 8
 
-dbg9IO :: Show a => String -> a -> IO ()
+dbg9IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg9IO = tracePrettyAtIO 9
 
 -- | Pretty-print a message and a showable value to the console if the debug level is at or above the specified level.
@@ -156,11 +152,9 @@ dbg9IO = tracePrettyAtIO 9
 tracePrettyAt :: Show a => Int -> String -> a -> a
 tracePrettyAt lvl = dbgppshow lvl
 
-tracePrettyAtIO :: Show a => Int -> String -> a -> IO ()
-tracePrettyAtIO lvl lbl x = tracePrettyAt lvl lbl x `seq` return ()
-
--- XXX
--- Could not deduce (a ~ ())
+-- tracePrettyAtM :: (Monad m, Show a) => Int -> String -> a -> m a
+-- tracePrettyAtM lvl lbl x = tracePrettyAt lvl lbl x `seq` return x
+-- XXX Could not deduce (a ~ ())
 -- from the context (Show a)
 --   bound by the type signature for
 --              dbgM :: Show a => String -> a -> IO ()
@@ -170,25 +164,25 @@ tracePrettyAtIO lvl lbl x = tracePrettyAt lvl lbl x `seq` return ()
 --       at hledger/Hledger/Cli/Main.hs:200:13
 -- Expected type: String -> a -> IO ()
 --   Actual type: String -> a -> IO a
---
--- tracePrettyAtM :: (Monad m, Show a) => Int -> String -> a -> m a
--- tracePrettyAtM lvl lbl x = tracePrettyAt lvl lbl x `seq` return x
+
+tracePrettyAtIO :: (MonadIO m, Show a) => Int -> String -> a -> m ()
+tracePrettyAtIO lvl lbl x = liftIO $ tracePrettyAt lvl lbl x `seq` return ()
 
 -- | print this string to the console before evaluating the expression,
--- if the global debug level is non-zero.  Uses unsafePerformIO.
-dbgtrace :: String -> a -> a
-dbgtrace
-    | debugLevel > 0 = trace
-    | otherwise      = flip const
+-- if the global debug level is at or above the specified level.  Uses unsafePerformIO.
+-- dbgtrace :: Int -> String -> a -> a
+-- dbgtrace level
+--     | debugLevel >= level = trace
+--     | otherwise           = flip const
 
 -- | Print a showable value to the console, with a message, if the
 -- debug level is at or above the specified level (uses
 -- unsafePerformIO).
 -- Values are displayed with show, all on one line, which is hard to read.
-dbgshow :: Show a => Int -> String -> a -> a
-dbgshow level
-    | debugLevel >= level = ltrace
-    | otherwise           = flip const
+-- dbgshow :: Show a => Int -> String -> a -> a
+-- dbgshow level
+--     | debugLevel >= level = ltrace
+--     | otherwise           = flip const
 
 -- | Print a showable value to the console, with a message, if the
 -- debug level is at or above the specified level (uses
@@ -218,16 +212,22 @@ dbgppshow level
 --                               return a
 --     | otherwise           = a
 
-
 -- | Like dbg, then exit the program. Uses unsafePerformIO.
 dbgExit :: Show a => String -> a -> a
-dbgExit msg = const (unsafePerformIO exitFailure) . dbg msg
+dbgExit msg = const (unsafePerformIO exitFailure) . dbg0 msg
 
 -- | Print a message and parsec debug info (parse position and next
 -- input) to the console when the debug level is at or above
 -- this level. Uses unsafePerformIO.
 -- pdbgAt :: GenParser m => Float -> String -> m ()
-pdbg :: Stream [Char] m t => Int -> String -> ParsecT [Char] st m ()
+pdbg :: Int -> String -> TextParser m ()
 pdbg level msg = when (level <= debugLevel) $ ptrace msg
 
-
+-- | Like dbg, but writes the output to "debug.log" in the current directory.
+-- Uses unsafePerformIO. Can fail due to log file contention if called too quickly
+-- ("*** Exception: debug.log: openFile: resource busy (file is locked)").
+dbglog :: Show a => String -> a -> a
+dbglog label a =
+  (unsafePerformIO $
+    appendFile "debug.log" $ label ++ ": " ++ ppShow a ++ "\n")
+  `seq` a
